@@ -50,9 +50,11 @@ All about the nature and implementation of the display algorithm is in
 
 import os
 import types
+from textwrap import dedent
 
 from ..parameters import DOWNLOADS, SERVER_DISPLAY, SERVER_DISPLAY_BASE
 from ..core.helpers import mdEsc, normpath, abspath, expanduser
+from ..core.timestamp import SILENT_D, silentConvert
 from .helpers import getRowsX, tupleEnum, RESULT, dh, showDict, _getLtr
 from .condense import condense, condenseSet
 from .highlight import getTupleHighlights
@@ -64,7 +66,7 @@ LIMIT_SHOW = 100
 LIMIT_TABLE = 2000
 
 
-def displayApi(app, silent):
+def displayApi(app, silent=SILENT_D):
     """Produce the display API.
 
     The display API provides methods to generate styled representations
@@ -84,12 +86,13 @@ def displayApi(app, silent):
     ----------
     app: obj
         The high-level API object
-    silent:
-        The verbosity mode to perform this operation in.
-        Normally it is the same as for the app, but when we do an `A.reuse()`
-        we force `silent=True`.
+    silent: string, optional `tf.core.timestamp.SILENT_D`
+        See `tf.core.timestamp.Timestamp`
+        Normally this parameter is taken from the app,
+        but when we do an `A.reuse()` we force `silent="deep"`.
     """
 
+    silent = silentConvert(silent)
     app.export = types.MethodType(export, app)
     app.table = types.MethodType(table, app)
     app.plainTuple = types.MethodType(plainTuple, app)
@@ -125,10 +128,11 @@ def displayShow(app, *options):
     tf.advanced.settings: options allowed in `config.yaml`
     """
 
+    inNb = app.inNb
     display = app.display
     display.setup()
     data = display.current
-    showDict("<b>current display options</b>", data, *options)
+    showDict("<b>current display options</b>", data, inNb, *options)
 
 
 def displaySetup(app, **options):
@@ -194,12 +198,15 @@ def loadCss(app):
     need to respond with the app-specific CSS: we return it as string.
     The flag `app._browse` is used to steer us into this case.
 
-    Otherwise, we collect the complete CSS code from Text-Fabric and the app,
+    Otherwise, if we are in a notebook,
+    we collect the complete CSS code from Text-Fabric and the app,
     and we add a piece to override some of the notebook CSS for tables,
     which specify a table layout with right aligned cell contents by default.
 
     We then load the resulting CSS into the notebook.
 
+    Otherwise, we do nothing.
+
     Returns
     -------
     None | string
@@ -214,8 +221,26 @@ def loadCss(app):
     if _browse:
         return appCss
 
+    if not app.inNb:
+        return
+
     css = getCss(app)
     dh(css)
+    dh(
+        dedent(
+            """
+            <script>
+            const copyChar = (el, c) => {
+                for (const el of document.getElementsByClassName('ccon')) {
+                    el.className = 'ccoff'
+                }
+                el.className = 'ccon'
+                navigator.clipboard.writeText(String.fromCharCode(c))
+            }
+            </script>
+            """
+        )
+    )
 
 
 def getCss(app):
@@ -256,9 +281,9 @@ def export(app, tuples, toDir=None, toFile="results.tsv", **options):
 
     !!! hint "Examples"
         See for detailed examples the
-        [exportExcel (bhsa)](https://nbviewer.jupyter.org/github/annotation/tutorials/blob/master/bhsa/exportExcel.ipynb)
+        [exportExcel (etcbc/bhsa)](https://nbviewer.jupyter.org/github/etcbc/bhsa/blob/master/tutorial/exportExcel.ipynb)
         and
-        [exportExcel (oldbabylonian)](https://nbviewer.jupyter.org/github/annotation/tutorials/blob/master/oldbabylonian/exportExcel.ipynb)
+        [exportExcel (Nino-cunei/oldbabylonian)](https://nbviewer.jupyter.org/github/Nino-cunei/oldbabylonian/blob/master/tutorial/exportExcel.ipynb)
         notebooks.
 
     Parameters
@@ -360,7 +385,7 @@ def export(app, tuples, toDir=None, toFile="results.tsv", **options):
     if toDir is None:
         toDir = expanduser(DOWNLOADS)
     else:
-        toDir = normpath(toDir)
+        toDir = expanduser(toDir)
     if not os.path.exists(toDir):
         os.makedirs(toDir, exist_ok=True)
     toPath = f"{toDir}/{toFile}"
@@ -411,6 +436,7 @@ def table(app, tuples, _asString=False, **options):
         return ""
 
     _browse = app._browse
+    inNb = app.inNb
 
     api = app.api
     F = api.F
@@ -444,7 +470,7 @@ def table(app, tuples, _asString=False, **options):
 
     newOptions = display.consume(options, "skipCols")
 
-    for (i, tup) in tupleEnum(tuples, start, end, LIMIT_TABLE, item):
+    for (i, tup) in tupleEnum(tuples, start, end, LIMIT_TABLE, item, inNb):
         if one:
             heads = '</th><th class="tf">'.join(fOtypev(n) for n in tup)
             html.append(
@@ -458,7 +484,7 @@ def table(app, tuples, _asString=False, **options):
             plainTuple(
                 app,
                 tup,
-                i,
+                seq=i,
                 item=item,
                 position=None,
                 opened=False,
@@ -471,11 +497,18 @@ def table(app, tuples, _asString=False, **options):
 
     if _browse or _asString:
         return html
-    dh(html)
+    dh(html, inNb=inNb)
 
 
 def plainTuple(
-    app, tup, seq, item=RESULT, position=None, opened=False, _asString=False, **options
+    app,
+    tup,
+    seq=None,
+    item=RESULT,
+    position=None,
+    opened=False,
+    _asString=False,
+    **options,
 ):
     """Display the plain text of a tuple of nodes.
 
@@ -487,10 +520,11 @@ def plainTuple(
     ----------
     tup: iterable of integer
         The members of the tuple can be arbitrary nodes.
-    seq: integer
+    seq: integer, optional `None`
         an arbitrary number which will be displayed in the first cell.
         This prepares the way for displaying query results, which come as
         a sequence of tuples of nodes.
+        If None, no such number is displayed in the heading.
     item: string, optional `result`
         A name for the tuple: it could be a result, or a chapter, or a line.
     position: integer, optional `None`
@@ -534,6 +568,7 @@ def plainTuple(
     otypeRank = N.otypeRank
     fOtypev = F.otype.v
     _browse = app._browse
+    inNb = app.inNb
 
     dContext = display.distill(options)
     condenseType = dContext.condenseType
@@ -570,11 +605,11 @@ def plainTuple(
 
     if _browse:
         prettyRep = (
-            prettyTuple(app, tup, seq, withPassage=False, **newOptions)
+            prettyTuple(app, tup, seq=seq, withPassage=False, **newOptions)
             if opened
             else ""
         )
-        current = "focus" if seq == position else ""
+        current = "focus" if seq is not None and seq == position else ""
         attOpen = "open " if opened else ""
         tupSeq = ",".join(str(n) for n in tup)
         if withPassage is True:
@@ -599,12 +634,16 @@ def plainTuple(
             + "</span>"
             for (i, n) in enumerate(tup)
         )
+        seqNo = -1 if seq is None else seq
+        seqRep = (
+            "" if seq is None else f'<a href="#" class="sq" tup="{tupSeq}">{seq}</a>'
+        )
         html = (
-            f'<details class="pretty dtrow {current}" seq="{seq}" {attOpen}>'
+            f'<details class="pretty dtrow {current}" seq="{seqNo}" {attOpen}>'
             f"<summary>"
             f'<a href="#" class="pq fa fa-solar-panel fa-xs"'
             f' title="show in context" {passageAtt}></a>'
-            f'<a href="#" class="sq" tup="{tupSeq}">{seq}</a>'
+            f"{seqRep}"
             f" {passageRef} {plainRep}"
             f"</summary>"
             f'<div class="pretty">{prettyRep}</div>'
@@ -612,7 +651,7 @@ def plainTuple(
         )
         return html
 
-    html = [str(seq)]
+    html = [] if seq is None else [str(seq)]
     if withPassage is True:
         html.append(passageRef)
     for (i, n) in enumerate(tup):
@@ -645,7 +684,7 @@ def plainTuple(
     )
     html = "<table>" + head + "".join(html) + "</table>"
 
-    dh(html)
+    dh(html, inNb=inNb)
 
 
 def plain(app, n, _inTuple=False, _asString=False, explain=False, **options):
@@ -721,6 +760,7 @@ def show(app, tuples, _asString=False, **options):
         return ""
 
     _browse = app._browse
+    inNb = app.inNb
     asString = _browse or _asString
 
     dContext = display.distill(options)
@@ -748,12 +788,12 @@ def show(app, tuples, _asString=False, **options):
 
     html = []
 
-    for (i, tup) in tupleEnum(tuples, start, end, LIMIT_SHOW, item):
+    for (i, tup) in tupleEnum(tuples, start, end, LIMIT_SHOW, item, inNb):
         item = F.otype.v(tup[0]) if condensed and condenseType else RESULT
         thisResult = prettyTuple(
             app,
             tup,
-            i,
+            seq=i,
             item=item,
             skipCols=set(),
             _asString=asString,
@@ -766,7 +806,7 @@ def show(app, tuples, _asString=False, **options):
         return "".join(html)
 
 
-def prettyTuple(app, tup, seq, _asString=False, item=RESULT, **options):
+def prettyTuple(app, tup, seq=None, _asString=False, item=RESULT, **options):
     """Displays the material that corresponds to a tuple of nodes in a graphical way.
 
     The member nodes of the tuple will be collected into containers, which
@@ -777,10 +817,11 @@ def prettyTuple(app, tup, seq, _asString=False, item=RESULT, **options):
     ----------
     tup: iterable of integer
         The members of the tuple can be arbitrary nodes.
-    seq: integer
+    seq: integer, optional `None`
         an arbitrary number which will be displayed in the heading.
         This prepares the way for displaying query results, which come as
         a sequence of tuples of nodes.
+        If None, no such number is displayed in the heading.
     item: string, optional `result`
         A name for the tuple: it could be a result, or a chapter, or a line.
     _asString: boolean, optional `False`
@@ -811,6 +852,7 @@ def prettyTuple(app, tup, seq, _asString=False, item=RESULT, **options):
     skipCols = dContext.skipCols
 
     _browse = app._browse
+    inNb = app.inNb
     asString = _browse or _asString
 
     if skipCols:
@@ -828,9 +870,10 @@ def prettyTuple(app, tup, seq, _asString=False, item=RESULT, **options):
 
     containers = {tup[0]} if condensed else condenseSet(api, tup, condenseType)
     highlights = getTupleHighlights(api, tup, highlights, colorMap, condenseType)
+    seqRep = "" if seq is None else f" <i>{seq}</i>"
 
     if not asString:
-        dh(f"<p><b>{item}</b> <i>{seq}</i></p>")
+        dh(f"<p><b>{item}</b>{seqRep}", inNb=inNb)
     if asString:
         html = []
     for t in sorted(containers, key=sortKey):
