@@ -1,9 +1,10 @@
 import os
+import re
 
 from IPython.display import display, Markdown, HTML
 
-from ..parameters import EXPRESS_BASE, GH_BASE, TEMP_DIR
-from ..core.helpers import mdEsc, htmlEsc, expanduser, unexpanduser, QUAD
+from ..parameters import backendRep, TEMP_DIR
+from ..core.helpers import mdEsc, htmlEsc, expanduser, unexpanduser, QUAD, console
 from ..core.text import DEFAULT_FORMAT
 
 
@@ -18,6 +19,24 @@ SEQ_TYPES1 = {tuple, list}
 SEQ_TYPES2 = {tuple, list, set, frozenset}
 
 
+def runsInNotebook():
+    """Determines whether the program runs in an interactive shell.
+
+    From https://stackoverflow.com/questions/15411967/how-can-i-check-if-code-is-executed-in-the-ipython-notebook/24937408
+    """
+    try:
+        runcontext = get_ipython()
+        shell = runcontext.__class__.__name__
+        if shell == "ZMQInteractiveShell":
+            return True  # Jupyter notebook or qtconsole
+        elif shell == "TerminalInteractiveShell":
+            return False  # Terminal running IPython
+        else:
+            return False  # Other type (?)
+    except NameError:
+        return False  # Probably standard Python interpreter
+
+
 def _getLtr(app, options):
     aContext = app.context
     direction = aContext.direction
@@ -31,73 +50,134 @@ def _getLtr(app, options):
     )
 
 
-def dm(md):
-    """Display markdown in a Jupyter notebook.
+def dm(md, inNb=True):
+    """Display markdown.
 
     Parameters
     ----------
     md: string
         Raw markdown string.
+    inNb: boolean, optional True
+        Whether the program runs in a notebook
 
     Returns
     -------
     None
-        The formatted markdown is rendered in the output cell.
+        The formatted markdown is rendered in the output cell if `inNb`
+        else the raw markdown is printed to the output.
     """
 
-    display(Markdown(unexpanduser(md)))
+    if inNb:
+        display(Markdown(unexpanduser(md)))
+    else:
+        console(md)
 
 
-def dh(html):
-    """Display HTML in a Jupyter notebook.
+def dh(html, inNb=True):
+    """Display HTML.
 
     Parameters
     ----------
     html: string
         Raw html string.
+    inNb: boolean, optional True
+        Whether the program runs in a notebook
 
     Returns
     -------
     None
-        The formatted HTML is rendered in the output cell.
+        The formatted HTML is rendered in the output cell if `inNb`
+        else the raw HTML is printed to the output.
     """
 
-    display(HTML(unexpanduser(html)))
+    if inNb:
+        display(HTML(unexpanduser(html)))
+    else:
+        console(html)
+
+
+# MODULE REFERENCES
+
+BACKEND_RE = re.compile(r"<([^/>]*)>")
+
+
+thisBackend = []
+
+
+def backendRepl(match):
+    thisBackend.append(match.group(1))
+    return ""
+
+
+def splitModRef(moduleRef):
+    thisBackend.clear()
+    theBackend = None
+    bareModuleRef = BACKEND_RE.sub(backendRepl, moduleRef)
+    if len(thisBackend):
+        theBackend = thisBackend[0]
+        if len(thisBackend) > 1:
+            console(
+                f"Multiple <backend> in {moduleRef}: "
+                f"{', '.join(thisBackend)}; using <{theBackend}> only ",
+                error=True,
+            )
+
+    bRep = f"<{theBackend}>" if theBackend else ""
+
+    parts = bareModuleRef.split(":", 1)
+    if len(parts) == 1:
+        parts.append("")
+    (ref, specifier) = parts
+    parts = ref.split("/", 2)
+
+    if len(parts) < 2:
+        console(
+            f"""
+Module ref "{bRep}{bareModuleRef}" is not "{{org}}/{{repo}}/{{path}}"
+""",
+            error=True,
+        )
+        return None
+
+    if len(parts) == 2:
+        parts.append("")
+
+    return [*parts, specifier, theBackend]
 
 
 # COLLECT CONFIG SETTINGS IN A DICT
 
 
-def getLocalDir(cfg, local, version):
+def getLocalDir(backend, cfg, local, version):
     provenanceSpec = cfg.get("provenanceSpec", {})
     org = provenanceSpec.get("org", None)
     repo = provenanceSpec.get("repo", None)
     relative = provenanceSpec.get("relative", "tf")
     version = provenanceSpec.get("version", None) if version is None else version
-    base = hasData(local, org, repo, version, relative)
+    base = hasData(backend, local, org, repo, version, relative)
 
     if not base:
-        base = EXPRESS_BASE
+        base = backendRep(backend, "cache")
 
     return expanduser(f"{base}/{org}/{repo}/{TEMP_DIR}")
 
 
-def hasData(local, org, repo, version, relative):
+def hasData(backend, local, org, repo, version, relative):
     versionRep = f"/{version}" if version else ""
     if local == "clone":
-        ghBase = expanduser(GH_BASE)
-        ghTarget = f"{ghBase}/{org}/{repo}/{relative}{versionRep}"
+        cloneBase = backendRep(backend, "clone")
+        ghTarget = f"{cloneBase}/{org}/{repo}/{relative}{versionRep}"
         if os.path.exists(ghTarget):
-            return ghBase
+            return cloneBase
 
-    expressBase = expanduser(EXPRESS_BASE)
-    expressTarget = f"{expressBase}/{org}/{repo}/{relative}{versionRep}"
-    if os.path.exists(expressTarget):
-        return expressBase
+    cacheBase = backendRep(backend, "cache")
+    cacheTarget = f"{cacheBase}/{org}/{repo}/{relative}{versionRep}"
+    if os.path.exists(cacheTarget):
+        return cacheBase
     return False
 
 
-def tupleEnum(tuples, start, end, limit, item):
+def tupleEnum(tuples, start, end, limit, item, inNb):
     if start is None:
         start = 1
     i = -1
@@ -123,7 +203,8 @@ def tupleEnum(tuples, start, end, limit, item):
         if rest:
             dh(
                 f"<b>{rest} more {item}s skipped</b> because we show a maximum of"
-                f" {limit} {item}s at a time"
+                f" {limit} {item}s at a time",
+                inNb=inNb,
             )
 
 
@@ -237,6 +318,7 @@ def getText(
     suppress = set() if options is None else options.suppress
 
     (tpl, feats) = templates[nType]
+    x = "" if isPretty else " "
 
     tplFilled = (
         (
@@ -263,8 +345,11 @@ def getText(
         )
         if tpl is True
         else (
-            tpl.format(
-                **{feat: getValue(app, n, nType, feat, suppress) for feat in feats}
+            (
+                tpl.format(
+                    **{feat: getValue(app, n, nType, feat, suppress) for feat in feats}
+                )
+                + x
             )
             if standardFeatures
             else ""
@@ -533,11 +618,13 @@ def eData(x, level):
     return eRest(x, level)
 
 
-def showDict(title, data, *keys):
+def showDict(title, data, inNb, *keys):
     """Shows selected keys of a dictionary in a pretty way.
 
     Parameters
     ----------
+    inNb: boolean
+        Whether we run in a notebook.
     keys: iterable of string
         For each key passed to this function, the information for that key
         will be displayed. If no keys are passed, all keys will be displayed.
@@ -561,4 +648,4 @@ def showDict(title, data, *keys):
             f"<summary>{i + 1}. {k}</summary>\n\n{eData(v, 0)}\n</details>\n"
         )
     md.append("</details>\n")
-    dm("".join(md))
+    dm("".join(md), inNb=inNb)
