@@ -2,19 +2,22 @@
 Produce links to Text-Fabric data and links from nodes to web resources.
 """
 
+import os
 import re
 import types
 from textwrap import dedent
 
 from ..parameters import (
-    URL_GH,
+    backendRep,
     URL_TFDOC,
     SEARCHREF,
     APIREF,
-    APP_URL,
-    APP_NB_URL,
+    APP_APP,
+    GH,
+    GL,
 )
-from ..core.helpers import htmlEsc, unexpanduser as ux
+from ..core.helpers import console, htmlEsc, unexpanduser as ux
+from ..core.timestamp import SILENT_D, AUTO, TERSE, VERBOSE, silentConvert
 from .repo import Checkout
 from .helpers import dh
 from ..server.wrap import wrapProvenance
@@ -22,12 +25,8 @@ from ..server.wrap import wrapProvenance
 
 UNSUPPORTED = ""
 
-pathRe = re.compile(
-    r"^(.*/(?:github|text-fabric-data))/([^/]+)/([^/]+)/(.*)$", flags=re.I
-)
 
-
-def linksApi(app, silent):
+def linksApi(app, silent=SILENT_D):
     """Produce the link API.
 
     The link API provides methods to maps nodes to urls of web resources.
@@ -44,11 +43,15 @@ def linksApi(app, silent):
     ----------
     app: obj
         The high-level API object
-    silent:
-        The verbosity mode to perform this operation in.
-        Normally it is the same as for the app, but when we do an `A.reuse()`
-        we force `silent=True`.
+    silent: string, optional `tf.core.timestamp.SILENT_D`
+        See `tf.core.timestamp.Timestamp`
+        Normally the silent parameter is taken from the app,
+        but when we do an `A.reuse()` we force `silent="deep"`.
     """
+    inNb = app.inNb
+    _browse = app._browse
+    silent = silentConvert(silent)
+    backend = app.backend
     app.showProvenance = types.MethodType(showProvenance, app)
     app.header = types.MethodType(header, app)
     app.webLink = types.MethodType(webLink, app)
@@ -56,9 +59,11 @@ def linksApi(app, silent):
 
     aContext = app.context
     appName = aContext.appName
+    appRelative = aContext.relative
     appPath = aContext.appPath
     apiVersion = aContext.apiVersion
     docUrl = aContext.docUrl
+    org = aContext.org or ""
     repo = aContext.repo or ""
     version = aContext.version
     corpus = aContext.corpus
@@ -67,8 +72,10 @@ def linksApi(app, silent):
     charUrl = aContext.charUrl
     charText = aContext.charText
 
-    tutUrl = f"{APP_NB_URL}/{appName}/start.ipynb"
-    extraUrl = f"{APP_URL}/app-{appName}"
+    tutUrl = (
+        f"{backendRep(backend, 'urlnb')}/"
+        f"{org}/{repo}/blob/master/tutorial/start.ipynb"
+    )
     apiVersionRep = "" if apiVersion is None else f" v{apiVersion}"
 
     dataName = repo.upper()
@@ -81,13 +88,22 @@ def linksApi(app, silent):
             dataName += f" volume {volumeInfo}"
 
     dataLink = (
-        outLink(dataName, docUrl, f"provenance of {corpus}")
+        outLink(dataName, docUrl, f"provenance of {corpus}", asHtml=inNb or _browse)
         if isCompatible and repo is not None and docUrl
-        else "/".join(x for x in (appPath, appName, version) if x)
+        else "/".join(
+            (x or "") for x in (appPath.rsplit("/", 1)[0], appRelative, version)
+        )
+        if appName.startswith("app:")
+        else "/".join((x or "") for x in (appPath, appName, version))
     )
     charLink = (
         (
-            outLink("Character table", charUrl.format(tfDoc=URL_TFDOC), charText)
+            outLink(
+                "Character table",
+                charUrl.format(tfDoc=URL_TFDOC),
+                charText,
+                asHtml=inNb or _browse,
+            )
             if isCompatible
             else UNSUPPORTED
         )
@@ -100,6 +116,7 @@ def linksApi(app, silent):
                 "Feature docs",
                 featureBase.replace("<feature>", featurePage).format(version=version),
                 f"{repo.upper()} feature documentation",
+                asHtml=inNb or _browse,
             )
             if isCompatible and repo is not None and featureBase
             else UNSUPPORTED
@@ -107,8 +124,14 @@ def linksApi(app, silent):
         if isCompatible
         else UNSUPPORTED
     )
+    extraUrl = f"{backendRep(backend, 'url')}/{org}/{repo}/blob/master/{APP_APP}"
     appLink = (
-        outLink(f"app-{appName}{apiVersionRep}", extraUrl, f"{appName} TF-app")
+        outLink(
+            f"{org}/{repo}/app {apiVersionRep}",
+            extraUrl,
+            f"{appName} TF-app",
+            asHtml=inNb or _browse,
+        )
         if isCompatible and repo is not None
         else "no app configured"
     )
@@ -116,18 +139,25 @@ def linksApi(app, silent):
         f"Text-Fabric API {app.TF.version}",
         APIREF,
         "text-fabric-api",
+        asHtml=inNb or _browse,
     )
     tfsLink = (
         outLink(
             "Search Reference",
             SEARCHREF,
             "Search Templates Introduction and Reference",
+            asHtml=inNb or _browse,
         )
         if isCompatible
         else UNSUPPORTED
     )
     tutLink = (
-        outLink("App tutorial", tutUrl, "App tutorial in Jupyter Notebook")
+        outLink(
+            "App tutorial",
+            tutUrl,
+            "App tutorial in Jupyter Notebook",
+            asHtml=inNb or _browse,
+        )
         if isCompatible and repo is not None
         else UNSUPPORTED
     )
@@ -141,17 +171,27 @@ def linksApi(app, silent):
     app.tutLink = tutLink
 
     if not app._browse:
-        if not silent:
-            header(app)
+        if silent in {VERBOSE, AUTO, TERSE}:
+            header(app, allMeta=silent == VERBOSE)
 
 
-def header(app):
+def header(app, allMeta=False):
     """Generate a colofon of the TF-app.
 
     This colofon will be displayed after initializing the advanced API,
     and it is packed with provenance and documentation links.
+
+    Parameters
+    ----------
+    allMeta: boolean, optional False
+        If True, includes all metadata of all features. This leads to big
+        stretches of largely redundant information in html details elements.
+        It is not visually cumbersome, but notebooks may grow excessively
+        if you load many datasets many times. So, if False, it will suppress
+        all that metadata except the description keys.
     """
 
+    inNb = app.inNb
     appLink = app.appLink
     dataLink = app.dataLink
     charLink = app.charLink
@@ -175,13 +215,24 @@ def header(app):
             '<img class="hdlogo" src="/server/static/icon.png"/>',
         )
     else:
-        tfLine = ", ".join(x for x in (tfLink, appLink, tfsLink) if x)
-        dataLine = ", ".join(x for x in (dataLink, charLink, featureLink) if x)
-        dh(
-            f"<b>Text-Fabric:</b> {tfLine}<br>"
-            f"<b>Data:</b> {dataLine}<br>"
-            "<b>Features:</b><br>" + _featuresPerModule(app)
-        )
+        featureInfo = _featuresPerModule(app, allMeta=allMeta)
+        if inNb:
+            tfLine = ", ".join(x for x in (tfLink, appLink, tfsLink) if x)
+            dataLine = ", ".join(x for x in (dataLink, charLink, featureLink) if x)
+            dh(
+                f"<b>Text-Fabric:</b> {tfLine}<br>"
+                f"<b>Data:</b> {dataLine}<br>"
+                f"<b>Features:</b><br>{featureInfo}"
+            )
+        else:
+            tfLine = "\n\t".join(x for x in (tfLink, appLink, tfsLink) if x)
+            dataLine = "\n\t".join(x for x in (dataLink, charLink, featureLink) if x)
+            console(
+                f"Text-Fabric:\n\t{tfLine}\n"
+                f"Data:\n\t{dataLine}\n"
+                f"Features:\n{featureInfo}\n",
+                newline=False,
+            )
 
 
 def webLink(
@@ -198,7 +249,7 @@ def webLink(
     !!! hint "Customizable"
         You can customize the behaviour of `webLink()` to the needs of your corpus
         by providing appropriate values in its `config.yaml`, especially for
-        `webBase`, `webLang`, `webOffset`, `webUrl`, `webUrlLex`, and `webHint`.
+        `webBase`, `webLang`, `webOffset`, `webFeature`, `webUrl`, `webUrlLex`, and `webHint`.
 
     Parameters
     ----------
@@ -226,14 +277,18 @@ def webLink(
     """
 
     api = app.api
+    TF = api.TF
     T = api.T
     F = api.F
     Fs = api.Fs
 
+    inNb = app.inNb
+    _browse = app._browse
     aContext = app.context
     webBase = aContext.webBase
     webLang = aContext.webLang
     webOffset = aContext.webOffset
+    webFeature = aContext.webFeature
     webUrl = aContext.webUrl
     webUrlZeros = aContext.webUrlZeros or {}
     webUrlLex = aContext.webUrlLex
@@ -262,28 +317,35 @@ def webLink(
         else:
             href = None
     else:
-        if webUrl:
-            href = webUrl
-            headingTuple = T.sectionFromNode(n, lang=webLang, fillup=True)
-            for (i, heading) in enumerate(headingTuple):
-                defaultOffset = 0 if type(heading) is int else ""
-                offset = (
-                    defaultOffset
-                    if webOffset is None
-                    else webOffset.get(i + 1, {}).get(
-                        headingTuple[i - 1], defaultOffset
+        href = None
+        if webFeature:
+            if TF.isLoaded(webFeature, pretty=False)[webFeature]:
+                val = Fs(webFeature).v(n)
+                if val is not None:
+                    href = f"{webBase}{val}"
+        if href is None:
+            if webUrl:
+                href = webUrl
+                headingTuple = T.sectionFromNode(n, lang=webLang, fillup=True)
+                for (i, heading) in enumerate(headingTuple):
+                    defaultOffset = 0 if type(heading) is int else ""
+                    offset = (
+                        defaultOffset
+                        if webOffset is None
+                        else webOffset.get(i + 1, {}).get(
+                            headingTuple[i - 1], defaultOffset
+                        )
+                        if i > 0
+                        else defaultOffset
                     )
-                    if i > 0
-                    else defaultOffset
-                )
-                value = "" if heading is None else str(heading + offset)
-                leadingZeros = webUrlZeros.get(i + 1, 0)
-                if 0 < len(value) < leadingZeros:
-                    value = "0" * (leadingZeros - len(value)) + value
+                    value = "" if heading is None else str(heading + offset)
+                    leadingZeros = webUrlZeros.get(i + 1, 0)
+                    if 0 < len(value) < leadingZeros:
+                        value = "0" * (leadingZeros - len(value)) + value
 
-                href = href.replace(f"<{i + 1}>", value)
-        else:
-            href = None
+                    href = href.replace(f"<{i + 1}>", value)
+            else:
+                href = None
 
     if nType in lexTypes:
         if text is None:
@@ -305,19 +367,20 @@ def webLink(
             href,
             clsName=clsName,
             passage=passageText,
+            asHtml=inNb or _browse,
             **atts,
         )
     result = href if urlOnly else fullResult
     if _asString or urlOnly:
         return result
-    dh(result)
+    dh(result, inNb=inNb)
 
 
 def showProvenance(app, jobName="program code", author="program author"):
     """Shows the provenance that is normally displayed during data loading.
 
-    This comes in handy if you have started with `use(xxx, silence='deep')` and still
-    need to show the provenance.
+    This comes in handy if you have started with
+    `use("org/repo", silence='deep')` and still need to show the provenance.
     Moreover, the provenance is shown in a formatted way.
 
     Parameters
@@ -332,10 +395,15 @@ def showProvenance(app, jobName="program code", author="program author"):
         This item will be displayed together with the rest of the provenance.
     """
 
+    inNb = app.inNb
     aContext = app.context
-    appName = aContext.appName
+    backend = app.backend
+    org = aContext.org
+    repo = aContext.repo
     commit = aContext.commit
-    appProvenance = ((("name", appName), ("commit", commit)),)
+    appProvenance = (
+        (("backend", backend), ("org", org), ("repo", repo), ("commit", commit)),
+    )
     provenance = (appProvenance, app.provenance)
     setNames = (
         tuple(sorted(app.sets.keys()))
@@ -343,10 +411,74 @@ def showProvenance(app, jobName="program code", author="program author"):
         else ()
     )
     form = dict(jobName=jobName, author=author)
-    dh(wrapProvenance(form, provenance, setNames)[0])
+    dh(wrapProvenance(form, provenance, setNames)[0], inNb=inNb)
 
 
-def outLink(text, href, title=None, passage=None, clsName=None, target="_blank"):
+PATH_RE1 = re.compile(
+    rf"""
+        ^
+        (.*?)
+        /
+        (?:
+            (
+                (?:
+                    text-fabric-data
+                    /
+                )?
+            )
+            (
+                {GH}
+                |
+                {GL}
+                |
+                (?:
+                    (?:
+                        [a-zA-Z0-9_-]+\.
+                    )+
+                    [a-zA-Z0-9_-]+
+                )
+            )
+        )
+        /
+        (.*)
+        $
+    """,
+    re.X | re.I,
+)
+
+PATH_RE2 = re.compile(
+    r"""
+    ^
+    ([^/]+)
+    /
+    ([^/]+)
+    /
+    (.*)
+    $
+    """,
+    re.X | re.I,
+)
+
+
+def _parseFeaturePath(path, backend):
+    match1 = PATH_RE1.fullmatch(path)
+    if not match1:
+        return (False, backend, "??", "??", "??")
+
+    (lead, exDir, theBackend, rest) = match1.groups()
+    theBackend = theBackend.lower()
+
+    match2 = PATH_RE2.fullmatch(rest)
+    if not match2:
+        return (False, theBackend, "??", "??", "??")
+
+    (org, repo, relative) = match2.groups()
+    return (True, theBackend, org, repo, relative)
+
+
+def outLink(
+    text, href, title=None, passage=None, clsName=None, target="_blank", asHtml=True
+):
     """Produce a formatted link.
 
     Parameters
@@ -364,6 +496,9 @@ def outLink(text, href, title=None, passage=None, clsName=None, target="_blank")
     passage: string, optional `None`
         A passage indicator, which will end up in the `sec` attribute of the
         link element. Used by the TF-browser.
+    asHtml: boolean, optional `True`
+        Whether we are in a notebook or in the browser. If not, a plain text representation
+        of the link will be made.
     """
 
     titleAtt = "" if title is None else f' title="{title}"'
@@ -371,13 +506,23 @@ def outLink(text, href, title=None, passage=None, clsName=None, target="_blank")
     targetAtt = f' target="{target}"' if target else ""
     passageAtt = f' sec="{passage}"' if passage else ""
     return (
-        f'<a{clsAtt}{targetAtt} href="{htmlEsc(href)}"{titleAtt}{passageAtt}>'
-        f"{text}</a>"
+        (
+            f'<a{clsAtt}{targetAtt} href="{htmlEsc(href)}"{titleAtt}{passageAtt}>'
+            f"{text}</a>"
+        )
+        if asHtml
+        else f"{text} => {href}"
     )
 
 
-def _featuresPerModule(app):
-    """Generate a formatted list of loaded TF features, per module."""
+def _featuresPerModule(app, allMeta=False):
+    """Generate a formatted list of loaded TF features, per module.
+
+    Parameters
+    ----------
+    allMeta: boolean, optional False
+        Whether to display all metadata for all features or the descriptions only
+    """
 
     isCompatible = app.isCompatible
     if isCompatible is not None and not isCompatible:
@@ -385,6 +530,8 @@ def _featuresPerModule(app):
 
     api = app.api
     TF = app.TF
+    inNb = app.inNb
+    backend = app.backend
 
     aContext = app.context
     mOrg = aContext.org
@@ -399,7 +546,9 @@ def _featuresPerModule(app):
 
     fixedModuleIndex = {}
     for m in moduleSpecs or []:
-        fixedModuleIndex[(m["org"], m["repo"], m["relative"])] = (
+        fixedModuleIndex[
+            (m["backend"] or backend, m["org"], m["repo"], m["relative"])
+        ] = (
             m["corpus"],
             m["docUrl"],
         )
@@ -409,12 +558,11 @@ def _featuresPerModule(app):
     baseLoc = mLocations[0] if hasattr(app, "mLocations") else ()
 
     for mLoc in mLocations:
-        match = pathRe.fullmatch(mLoc)
-        if not match:
-            moduleIndex[mLoc] = ("??", "??", "??", mLoc, "")
+        (parseOK, theBackend, org, repo, relative) = _parseFeaturePath(mLoc, backend)
+        if not parseOK:
+            moduleIndex[mLoc] = (theBackend, org, repo, relative, mLoc, "")
         else:
-            (base, org, repo, relative) = match.groups()
-            mId = (org, repo, relative)
+            mId = (theBackend, org, repo, relative)
             (corpus, docUrl) = (
                 (relative, None)
                 if org is None or repo is None
@@ -427,11 +575,11 @@ def _featuresPerModule(app):
                 else fixedModuleIndex[mId]
                 if mId in fixedModuleIndex
                 else (
-                    f"{org}/{repo}/{relative}",
-                    f"{URL_GH}/{org}/{repo}/tree/master/{relative}",
+                    f"{backendRep(backend, 'rep')}{org}/{repo}/{relative}",
+                    f"{backendRep(backend, 'url')}/{org}/{repo}/tree/master/{relative}",
                 )
             )
-            moduleIndex[mId] = (org, repo, relative, corpus, docUrl)
+            moduleIndex[mId] = (theBackend, org, repo, relative, corpus, docUrl)
 
     featureCat = {}
 
@@ -439,25 +587,30 @@ def _featuresPerModule(app):
         added = False
         featureInfo = TF.features[feature]
         featurePath = featureInfo.path
-        match = pathRe.fullmatch(featurePath)
-        if match:
-            (base, fOrg, fRepo, relative) = match.groups()
+
+        (parsedOK, fBackend, fOrg, fRepo, relative) = _parseFeaturePath(
+            os.path.dirname(featurePath), backend
+        )
+
+        if parsedOK:
             fRelative = relative.rsplit("/", 1)[0]
-            mId = (fOrg, fRepo, fRelative)
+            mId = (fBackend, fOrg, fRepo, fRelative)
         else:
             mId = featurePath.rsplit("/", 1)[0]
+
         if type(mId) is str:
-            for (mIId, mInfo) in moduleIndex.items():
+            for mIId in moduleIndex:
                 if type(mIId) is str:
                     if featurePath.startswith(mIId):
                         featureCat.setdefault(mIId, []).append(feature)
                         added = True
         else:
-            for (mIId, mInfo) in moduleIndex.items():
+            for mIId in moduleIndex:
                 if type(mIId) is not str:
-                    (mOrg, mRepo, mRelative) = mIId
+                    (mBackend, mOrg, mRepo, mRelative) = mIId
                     if (
-                        fOrg == mOrg
+                        fBackend == mBackend
+                        and fOrg == mOrg
                         and fRepo == mRepo
                         and fRelative.startswith(mRelative)
                     ):
@@ -466,7 +619,7 @@ def _featuresPerModule(app):
         if not added:
             featureCat.setdefault(mId, []).append(feature)
 
-    baseId = (mOrg, mRepo, mRelative)
+    baseId = (backend, mOrg, mRepo, mRelative)
     baseMods = {
         mId for mId in featureCat.keys() if type(mId) is tuple and mId == baseId
     }
@@ -475,26 +628,34 @@ def _featuresPerModule(app):
         key=lambda mId: (1, mId) if type(mId) is str else (0, mId),
     )
 
-    html = ""
+    output = ""
+
     for mId in moduleOrder:
         catFeats = featureCat[mId]
         if not catFeats:
             continue
         modInfo = moduleIndex.get(mId, None)
         if modInfo:
-            (org, repo, relative, corpus, docUrl) = modInfo
+            (mBackend, org, repo, relative, corpus, docUrl) = modInfo
         else:
             corpus = mId if type(mId) is str else "/".join(mId)
             docUrl = (
                 ""
                 if type(mId) is str
-                else f"{URL_GH}/{mId[0]}/{mId[1]}/tree/master/{mId[2]}"
+                else (
+                    f"{backendRep(mBackend, 'url')}/"
+                    f"{mId[0]}/{mId[1]}/tree/master/{mId[2]}"
+                )
             )
-        html += dedent(
-            f"""
+        output += (
+            dedent(
+                f"""
             <details><summary><b>{corpus}</b></summary>
                 <div class="fcorpus">
         """
+            )
+            if inNb
+            else f"{corpus}\n"
         )
 
         seen = set()
@@ -518,93 +679,145 @@ def _featuresPerModule(app):
             description = meta.get("description", "")
 
             edgeRep = "edge" if isEdge else ""
-            html += dedent(
-                f"""
-                    <div class="frow">
-                        <div class="fnamecat {edgeRep}">
-                """
-            )
-            html += (
-                outLink(
-                    featureRep,
-                    docUrl.replace("<feature>", featureRep),
-                    title=featurePath,
-                )
-                if docUrl
-                else f'<span title="{featurePath}">{featureRep}</span>'
-            )
-            html += dedent(
-                f"""
-                        </div>
-                        <div class="fmono">{typeRep}</div>
-                        <details>
-                            <summary>{description}</summary>
-                            <div class="fmeta">
-                """
-            )
-            for (k, v) in sorted(meta.items()):
-                if k not in {"valueType", "description"}:
-                    k = htmlEsc(k)
-                    v = htmlEsc(v)
-                    html += dedent(
-                        f"""
-                            <div class="fmetarow">
-                                <div class="fmetakey">{k}:</div>
-                                <div>{v}</div>
-                            </div>
+            if inNb:
+                output += dedent(
+                    f"""
+                        <div class="frow">
+                            <div class="fnamecat {edgeRep}">
                     """
+                )
+            output += (
+                (
+                    outLink(
+                        featureRep,
+                        docUrl.replace("<feature>", featureRep),
+                        title=featurePath,
+                        asHtml=True,
                     )
-            html += dedent(
-                """
+                    if docUrl
+                    else f'<span title="{featurePath}">{featureRep}</span>'
+                )
+                if inNb
+                else f"\t{featureRep:<20} "
+            )
+            if inNb:
+                output += dedent(
+                    f"""
                             </div>
-                        </details>
+                            <div class="fmono">{typeRep}</div>
+                    """
+                )
+            if inNb:
+                output += dedent(
+                    f"""
+                            <details>
+                                <summary>{description}</summary>
+                                <div class="fmeta">
+                    """
+                    if allMeta
+                    else f"""
+                            <span> {description}</span>
+                    """
+                )
+            else:
+                output += f"{description}\n"
+            if allMeta:
+                for (k, v) in sorted(meta.items()):
+                    if k not in {"valueType", "description"}:
+                        if inNb:
+                            k = htmlEsc(k)
+                            v = htmlEsc(v)
+                            output += dedent(
+                                f"""
+                                    <div class="fmetarow">
+                                        <div class="fmetakey">{k}:</div>
+                                        <div>{v}</div>
+                                    </div>
+                            """
+                            )
+                        else:
+                            output += f"\t\t{k}: {v}\n"
+                if inNb:
+                    output += dedent(
+                        """
+                                    </div>
+                                </details>
+                        """
+                    )
+                else:
+                    output += "\n"
+            if inNb:
+                output += dedent(
+                    """
+                        </div>
+                    """
+                )
+        if inNb:
+            output += dedent(
+                """
                     </div>
+                </details>
                 """
             )
-        html += dedent(
-            """
-                </div>
-            </details>
-            """
-        )
-    return html
+        else:
+            output += "\n"
+    return output
 
 
-def provenanceLink(org, repo, version, commit, release, local, relative):
+def provenanceLink(backend, org, repo, version, commit, local, release, relative):
     """Generate a provenance link for a data source.
 
-    We assume the data source resides somewhere inside a GitHub repo.
+    We assume the data source resides somewhere inside a backend repository.
 
     Parameters
     ----------
+    backend: string
+        `github` or `gitlab` or a GitLab instance such as `gitlab.huc.knaw.nl`.
     org: string
-        Organization on GitHub
+        Organization on GitHub or group on GitLab
     repo: string
-        Repository on Github
+        Repository on GitHub or project on GitLab
     version: string
         Version of the data source.
         This is not the release or commit of a repo, but the subdirectory
         corresponding with a data version under a `tf` directory with feature files.
     commit: string
-        The commit hash of the repository on GitHub.
+        The commit hash of the repository
+    local: boolean
+        Whether the data is on the local computer and not necessarily backed up
+        by a backend repository
+    release: string
+        The release tag of the repository
     """
 
     text = (
         f"data on local machine {relative}"
         if org is None or repo is None
-        else f"{org}/{repo} v:{version} ({Checkout.toString(commit, release, local)})"
+        else (
+            f"{org}/{repo} v:{version}"
+            f"({Checkout.toString(commit, release, local, backend)})"
+        )
     )
     relativeFlat = relative.replace("/", "-")
+    bUrl = backendRep(backend, "url")
     url = (
         None
         if org is None or repo is None
-        else f"{URL_GH}/{org}/{repo}/tree/master/{relative}"
+        else f"{bUrl}/{org}/{repo}/tree/master/{relative}"
         if local
         else (
-            f"{URL_GH}/{org}/{repo}/releases/download/{release}"
-            f"/{relativeFlat}-{version}.zip"
+            (
+                (
+                    f"{bUrl}/{org}/{repo}/releases/download/{release}"
+                    f"/{relativeFlat}-{version}.zip"
+                )
+                if backend == GH
+                else (
+                    f"{bUrl}/{org}/{repo}/-/archive/{release}" f"/{repo}-{version}.zip"
+                )
+            )
             if release
-            else f"{URL_GH}/{org}/{repo}/tree/{commit}/{relative}"
+            else f"{bUrl}/{org}/{repo}/tree/{commit}/{relative}"
         )
     )
     return (text, url)
