@@ -97,7 +97,7 @@ lineNumbers: boolean, optional `False`
 
         If the corpus has no line numbers, the default is `None`.
 
-plainGaps: boolean, optional `False`
+plainGaps: boolean, optional `True`
     **interface option**
     indicates whether gaps types should be displayed in plain displays.
     In pretty displays gaps are marked by dotted left-right borders of the nodes
@@ -445,12 +445,14 @@ class Options:
     def reset(self, *options):
         app = self.app
         error = app.error
+        # info = app.info
         defaults = self.defaults
 
         if options:
             current = self.current
             for option in options:
                 if option not in defaults:
+                    # info(defaults)
                     error(f'WARNING: unknown display option "{option}" will be ignored')
                     continue
                 current[option] = defaults[option]
@@ -493,14 +495,14 @@ class Options:
             if not value:
                 value = set()
             elif type(value) is str:
-                value = set(int(v) for v in value.split()) if value else set()
+                value = {int(v) for v in value.split()} if value else set()
             elif type(value) not in {set, frozenset}:
                 value = set(value)
         elif option in {"withPassage"}:
             if not value:
                 value = False
             elif type(value) is str:
-                value = set(int(v) for v in value.split()) if value else set()
+                value = {int(v) for v in value.split()} if value else set()
             elif type(value) in {list, tuple, dict}:
                 value = set(value)
             elif type(value) is not set:
