@@ -4,7 +4,8 @@ Calls from the advanced API to the Search API.
 
 import types
 
-from ..core.helpers import console
+from ..core.helpers import console, wrapMessages
+from ..core.timestamp import SILENT_D, silentConvert
 from .condense import condense
 
 
@@ -12,7 +13,7 @@ def searchApi(app):
     app.search = types.MethodType(search, app)
 
 
-def search(app, query, silent=False, sets=None, shallow=False, sort=True):
+def search(app, query, silent=SILENT_D, sets=None, shallow=False, sort=True, limit=None):
     """Search with some high-level features.
 
     This function calls the lower level `tf.search.search.Search` facility aka `S`.
@@ -33,8 +34,8 @@ def search(app, query, silent=False, sets=None, shallow=False, sort=True):
         the search template (`tf.about.searchusage`)
         that has to be searched for.
 
-    silent: boolean, optional `False`
-        if `True` it will suppress the reporting of the number of results.
+    silent: string, optional `tf.core.timestamp.SILENT_D`
+        See `tf.core.timestamp.Timestamp`
 
     shallow: boolean, optional `False`
         If `True` or `1`, the result is a set of things that match the top-level element
@@ -80,6 +81,16 @@ def search(app, query, silent=False, sets=None, shallow=False, sort=True):
 
         If it is a `False` value, no sorting will be applied.
 
+    limit: integer, optional `None`
+        If `limit` is a positive number, it will fetch only that many results.
+        If it is negative, 0, None, or absent, it will fetch arbitrary many results.
+
+        !!! caution "there is an upper *fail limit* for safety reasons.
+            The limit is a factor times the max node in your corpus.
+            See `tf.parameters.SEARCH_FAIL_FACTOR`.
+            If this *fail limit* is exceeded in cases where no positive `limit`
+            has been passed, you get a warning message.
+
     !!! hint "search template reference"
         See the search template reference (`tf.about.searchusage`)
 
@@ -89,7 +100,7 @@ def search(app, query, silent=False, sets=None, shallow=False, sort=True):
         Web apps can better use `tf.advanced.search.runSearch`.
     """
 
-    info = app.info
+    warning = app.warning
     isSilent = app.isSilent
     setSilent = app.setSilent
     api = app.api
@@ -99,7 +110,9 @@ def search(app, query, silent=False, sets=None, shallow=False, sort=True):
 
     wasSilent = isSilent()
 
-    results = S.search(query, sets=sets, shallow=shallow)
+    silent = silentConvert(silent)
+
+    results = S.search(query, sets=sets, shallow=shallow, limit=limit)
     if not shallow:
         if not sort:
             results = list(results)
@@ -133,7 +146,7 @@ def search(app, query, silent=False, sets=None, shallow=False, sort=True):
     nResults = len(results)
     plural = "" if nResults == 1 else "s"
     setSilent(silent)
-    info(f"{nResults} result{plural}")
+    warning(f"{nResults} result{plural}")
     setSilent(wasSilent)
     return results
 
@@ -170,8 +183,9 @@ def runSearch(app, query, cache):
             for (i, q) in enumerate(qnodes)
         )
     queryResults = tuple(sorted(queryResults))
-    cache[cacheKey] = (queryResults, messages, features)
-    return (queryResults, messages, features)
+    runMessages = wrapMessages(S._msgCache)
+    cache[cacheKey] = (queryResults, (messages, runMessages), features)
+    return (queryResults, (messages, runMessages), features)
 
 
 def runSearchCondensed(app, query, cache, condenseType):
