@@ -6,8 +6,8 @@ These settings will be read, checked, and transformed into configuration data
 that is read by the app, see `tf.advanced.settings.showContext`
 
 See for examples:
-* [bhsa](https://github.com/annotation/app-bhsa/blob/master/code/config.yaml).
-* [uruk](https://github.com/annotation/app-uruk/blob/master/code/config.yaml).
+* [bhsa](https://github.com/etcbc/bhsa/blob/master/app/config.yaml).
+* [uruk](https://github.com/Nino-cunei/uruk/blob/master/app/config.yaml).
 
 # Config specs
 
@@ -226,10 +226,11 @@ Default:
 
 ### `docRoot`
 
-Where the docs are: on Github (default), or on nbviewer (`{urlNb}`) or somewhere else.
+Where the docs are: on GitHub or GitLab (default),
+or on nbviewer (`{urlNb}`) or somewhere else.
 
 Default:
-:   string `{urlGh}`
+:   string `{urlGh}` or `https://`*backend*
 
 ---
 
@@ -320,6 +321,7 @@ as many as you want:
 ```
 moduleSpecs = (
   dict(
+      backend="gitlab",
       org="researcher1",
       repo="work1",
       relative="tf",
@@ -330,6 +332,7 @@ moduleSpecs = (
       doi="xx.yyyy/archive.zzzzzzz",
   ),
   dict(
+      backend="gitlab.huc.knaw.nl",
       org="researcher2",
       repo="work2",
       relative="tf",
@@ -353,7 +356,7 @@ Default:
 
 ### `org`
 
-The GitHub organisation name under which your TF data resides.
+The GitHub organisation or the GitLab group under which your TF data resides.
 
 Default:
 :   string `annotation`
@@ -372,7 +375,7 @@ Default:
 
 ### `repo`
 
-The GitHub repo name under which your TF data resides
+The GitHub repo or the GitLab project under which your TF data resides
 
 !!! hint
     *org/repo* = `annotation/default` acts as placeholder for app-less datasets.
@@ -400,6 +403,19 @@ Default:
 
 ---
 
+### `webFeature`
+
+If passed, contains the name of the feature that stores the part of the weblink for
+that node that comes after `webBase`.
+This overrides `webUrl` in that when `webFeature` is present, and a node has a value
+for it, than that value will be used in composing the web link, rather than filling
+the template `webLink` with values from the headings.
+
+Default:
+:   string `null`
+
+---
+
 ### `webHint`
 
 If passed, will be used as hint text when the user hovers over a web link
@@ -445,7 +461,7 @@ of the online resource (see `webUrl`).
 Suppose we need to offset sections of level 2 depending on the section of level 1
 they are in.
 For example, in the
-[missieven corpus](https://github.com/annotation/app-missieven/blob/master/code/config.yaml)
+[missieven corpus](https://github.com/clariah/wp6-missieven/blob/master/app/config.yaml)
 we have section levels 1=volume, 2=page, 3=line.
 In each volume, the logical page 1 must be translated into a higher
 number, depending on the number of preface pages in that volume.
@@ -473,13 +489,21 @@ Missing values translate to 0 or the empty string.
 If present, `webLink(node)` will use this as a template to generate a url
 to an online edition of the node.
 
-The following place holders will be honoured:
+This may happen in two ways:
 
-*   `{webBase}`: the webBase above
-*   `<1>` : value for section heading 1
-*   `<2>` : value for section heading 2
-*   `<3>` : value for section heading 3
-*   `{version}`: version of the TF resource
+*   From a feature whose name is given in `webFeature` we take the value for node `n`.
+    If that value exists, it will be appended to the value of `webBase` and that
+    will be the link.
+*   If `webFeature` is not given, or if it is given,
+    but the feature has no value for `n`, the web link will be computed from the
+    section headings.
+    The following place holders will be honoured:
+
+    *   `{webBase}`: the webBase above
+    *   `<1>` : value for section heading 1
+    *   `<2>` : value for section heading 2
+    *   `<3>` : value for section heading 3
+    *   `{version}`: version of the TF resource
 
 Default:
 :   string `null`
@@ -524,7 +548,7 @@ Default:
 ### `zip`
 
 Only used by `text-fabric-zip` when collecting data into zip files
-as attachments to a GitHub release.
+as attachments to a GitHub/GitLab release.
 
 If left to `null`, will be configured to use the main repo and the modules.
 
@@ -620,7 +644,7 @@ If there are no sections, we pick a medium-sized node type.
 If there are no such node types, we pick the slot type, but this is pathological.
 
 Default:
-:   boolean `true`
+:   boolean `false`
 
 ---
 
@@ -632,7 +656,7 @@ All nodes that satisfy at least one of these conditions will be left out.
 
 !!! hint
     Use this if you want to exclude particular nodes of some type, e.g. in
-    [dss](https://github.com/annotation/app-dss/blob/master/code/config.yaml).
+    [dss](https://github.com/etcbc/dss/blob/master/app/config.yaml).
     where we want to prevent line terminator signs.
 
 The value is a dictionary of feature name - value pairs.
@@ -696,7 +720,7 @@ The app needs to define a function
 getGraphics(isPretty, node, nodeType, isOuter) => HTML code for sourcing the graphics
 ```
 
-See [uruk](https://github.com/annotation/app-uruk/blob/master/code/app.py).
+See [uruk](https://github.com/Nino-cunei/uruk/blob/master/app/app.py).
 
 Default
 :   boolean `null`
@@ -803,7 +827,7 @@ Default
 
 !!! hint
     For some types in
-    [uruk](https://github.com/annotation/app-uruk/blob/master/code/config.yaml)
+    [uruk](https://github.com/Nino-cunei/uruk/blob/master/app/config.yaml)
     it is needed to deviate from the default.
 
 ---
@@ -813,7 +837,7 @@ Default
 Formatting key for plain and pretty displays.
 
 The label or template may need a special style to format it with.
-You pass specify:
+You can specify:
 
 `normal`
 :   normal non-corpus text
@@ -873,7 +897,7 @@ The default is:
 `true` for the lowest section type, if there are section types in `otext.tf`.
 
 But more types can be declared as verselike, e.g. `halfverse` in the
-[bhsa](https://github.com/annotation/app-bhsa/blob/master/code/config.yaml).
+[bhsa](https://github.com/etcbc/bhsa/blob/master/app/config.yaml).
 
 ---
 
@@ -890,7 +914,7 @@ Default:
 
 !!! hint
     For some types in
-    [uruk](https://github.com/annotation/app-uruk/blob/master/code/config.yaml)
+    [uruk](https://github.com/Nino-cunei/uruk/blob/master/app/config.yaml)
     it is needed to deviate from the default.
 
 ---
@@ -909,7 +933,7 @@ Default:
 import re
 import types
 
-from ..parameters import URL_GH, URL_NB, URL_TFDOC
+from ..parameters import backendRep, URL_TFDOC
 from ..core.helpers import console, mergeDictOfSets
 from .options import INTERFACE_OPTIONS
 from .helpers import parseFeatures, transitiveClosure, showDict, ORIG, NORMAL
@@ -918,14 +942,35 @@ from .helpers import parseFeatures, transitiveClosure, showDict, ORIG, NORMAL
 VAR_PATTERN = re.compile(r"\{([^}]+)\}")
 
 WRITING_DEFAULTS = dict(
-    akk=dict(language="akkadian", direction="ltr",),
-    hbo=dict(language="hebrew", direction="rtl",),
-    syc=dict(language="syriac", direction="rtl",),
-    ara=dict(language="arabic", direction="rtl",),
-    grc=dict(language="greek", direction="ltr",),
-    cld=dict(language="aramaic", direction="ltr",),
+    akk=dict(
+        language="akkadian",
+        direction="ltr",
+    ),
+    hbo=dict(
+        language="hebrew",
+        direction="rtl",
+    ),
+    syc=dict(
+        language="syriac",
+        direction="rtl",
+    ),
+    ara=dict(
+        language="arabic",
+        direction="rtl",
+    ),
+    grc=dict(
+        language="greek",
+        direction="ltr",
+    ),
+    cld=dict(
+        language="aramaic",
+        direction="ltr",
+    ),
+)
+WRITING_DEFAULTS[""] = dict(
+    language="",
+    direction="ltr",
 )
-WRITING_DEFAULTS[""] = dict(language="", direction="ltr",)
 
 FONT_BASE = (
     "https://github.com/annotation/text-fabric/blob/master/tf/server/static/fonts"
@@ -968,6 +1013,7 @@ RELATIVE_DEFAULT = "tf"
 
 MSPEC_KEYS = set(
     """
+    backend
     org
     repo
     relative
@@ -992,6 +1038,7 @@ PROVENANCE_DEFAULTS = (
     ("webLang", None),
     ("webLexId", None),
     ("webOffset", None),
+    ("webFeature", None),
     ("webUrl", None),
     ("webUrlZeros", None),
     ("webUrlLex", None),
@@ -999,17 +1046,20 @@ PROVENANCE_DEFAULTS = (
     ("webHint", None),
 )
 
-DOC_DEFAULTS = (
-    ("docRoot", "{urlGh}"),
-    ("docExt", ".md"),
-    ("docBase", "{docRoot}/{org}/{repo}/blob/master/docs"),
-    ("docPage", "home"),
-    ("docUrl", "{docBase}/{docPage}{docExt}"),
-    ("featureBase", "{docBase}/features/<feature>{docExt}"),
-    ("featurePage", "home"),
-    ("charUrl", "{tfDoc}/writing/{language}.html"),
-    ("charText", "How TF features represent text"),
-)
+
+def DOC_DEFAULTS(backend):
+    return (
+        ("docRoot", f"{backendRep(backend, 'url')}"),
+        ("docExt", ".md"),
+        ("docBase", "{docRoot}/{org}/{repo}/blob/master/docs"),
+        ("docPage", "home"),
+        ("docUrl", "{docBase}/{docPage}{docExt}"),
+        ("featureBase", "{docBase}/features/<feature>{docExt}"),
+        ("featurePage", "home"),
+        ("charUrl", "{tfDoc}/writing/{language}.html"),
+        ("charText", "How TF features represent text"),
+    )
+
 
 DATA_DISPLAY_DEFAULTS = (
     ("excludedFeatures", set(), False),
@@ -1272,9 +1322,14 @@ def _fillInDefined(template, data):
 
 
 def setAppSpecs(app, cfg, reset=False):
+    backend = app.backend
     if not reset:
         app.customMethods = AppCurrent({hook: {} for hook in HOOKS})
-    specs = dict(urlGh=URL_GH, urlNb=URL_NB, tfDoc=URL_TFDOC,)
+    specs = dict(
+        urlGh=backendRep(backend, "rep"),
+        urlNb=backendRep(backend, "urlnb"),
+        tfDoc=URL_TFDOC,
+    )
     app.specs = specs
     specs.update(cfg)
     if "apiVersion" not in specs:
@@ -1294,7 +1349,7 @@ def setAppSpecs(app, cfg, reset=False):
 
     for (dKey, defaults) in (
         ("provenanceSpec", PROVENANCE_DEFAULTS),
-        ("docs", DOC_DEFAULTS),
+        ("docs", DOC_DEFAULTS(backend)),
     ):
         checker.checkGroup(cfg, {d[0] for d in defaults}, dKey)
         checker.report()
@@ -1324,7 +1379,7 @@ def setAppSpecs(app, cfg, reset=False):
                     else:
                         moduleSpec[k] = (
                             specs.get(k, None)
-                            if k in {"org", "repo"}
+                            if k in {"backend", "org", "repo"}
                             else RELATIVE_DEFAULT
                             if k == "relative"
                             else None
@@ -1339,7 +1394,7 @@ def setAppSpecs(app, cfg, reset=False):
         graphicsModule = [(org, repo, graphicsRelative)] if graphicsRelative else []
         specs["zip"] = (
             [repo]
-            + [(m["org"], m["repo"], m["relative"],) for m in moduleSpecs]
+            + [(m["backend"], m["org"], m["repo"], m["relative"]) for m in moduleSpecs]
             + graphicsModule
         )
 
@@ -1542,7 +1597,10 @@ def getTypeDefaults(app, cfg, dKey, withApi):
             baseTypes.add(nType)
 
         if "condense" in info:
-            condenseType = nType
+            condense = info["condense"]
+            checker.checkSetting("condense", condense)
+            if condense:
+                condenseType = nType
 
         trans = info.get("transform", None)
         if trans is not None:
@@ -1562,7 +1620,9 @@ def getTypeDefaults(app, cfg, dKey, withApi):
                 )
                 dest[nType] = (template, templateFeatures)
                 checker.checkSetting(
-                    k, template, extra=(template, templateFeatures),
+                    k,
+                    template,
+                    extra=(template, templateFeatures),
                 )
 
         if "style" in info:
@@ -1725,7 +1785,9 @@ def getTypeDefaults(app, cfg, dKey, withApi):
         )
 
         levelCls[nType] = dict(
-            container=containerCls, label=labelCls, children=childrenCls,
+            container=containerCls,
+            label=labelCls,
+            children=childrenCls,
         )
 
     descendantType = transitiveClosure(childType, {slotType})
@@ -1777,7 +1839,8 @@ def showContext(app, *keys):
     tf.advanced.settings: options allowed in `config.yaml`
     """
 
-    showDict(f"<b>{(app.appName)}</b> <i>app context</i>", app.specs, *keys)
+    inNb = app.inNb
+    showDict(f"<b>{(app.appName)}</b> <i>app context</i>", app.specs, inNb, *keys)
 
 
 def getLevel(defaultLevel, givenInfo, isVerse):
