@@ -51,7 +51,7 @@ def compose(
             prettyTuple(
                 app,
                 tup,
-                getx,
+                seq=getx,
                 extraFeatures=(features, {}),
                 **display.consume(options, "extraFeatures"),
             )
@@ -84,7 +84,7 @@ def compose(
             plainTuple(
                 app,
                 tup,
-                i,
+                seq=i,
                 item=item,
                 position=position,
                 opened=i in opened,
@@ -132,7 +132,7 @@ def composeT(
             prettyTuple(
                 app,
                 tup,
-                getx,
+                seq=getx,
                 condensed=False,
                 extraFeatures=(features, {}),
                 **display.consume(options, "condensed", "extraFeatures"),
@@ -164,7 +164,7 @@ def composeT(
             plainTuple(
                 app,
                 tup,
-                i,
+                seq=i,
                 condensed=False,
                 extraFeatures=(features, {}),
                 opened=i in opened,
@@ -197,6 +197,15 @@ def composeP(
 
     Typically invoked when a user of the TF-browser is browsing passages.
     The query is used to highlight its results in the passages that the user is browsing.
+
+    Parameters
+    ----------
+    items: int or list of int
+        A node or list of nodes.
+        Some corpora require a passage to be shown as a pretty display of the section node
+        in question, and then items is a single node.
+        Normally, the items is the list of section nodes one level lower than the
+        section node in question. See *browseContentPretty* in `tf.advanced.settings`.
     """
 
     display = app.display
@@ -214,7 +223,7 @@ def composeP(
         return ""
 
     if type(items) is int:
-        return pretty(app, items)
+        return pretty(app, items, **options)
 
     if getx is not None:
         tup = None
@@ -227,7 +236,7 @@ def composeP(
             prettyTuple(
                 app,
                 tup,
-                getx,
+                seq=getx,
                 condensed=False,
                 condenseType=finalSecType,
                 extraFeatures=(features, {}),
@@ -286,7 +295,7 @@ def _plainTextSFinal(
         prettyTuple(
             app,
             (sNode,),
-            secStr,
+            sec=secStr,
             condensed=False,
             condenseType=finalSecType,
             **display.consume(options, "condensed", "condenseType"),
