@@ -8,16 +8,19 @@ Text-Fabric apps.
 Such a search interface is a static HTML page, powered by a Javascript program
 that reads the corpus data from Javascript variables.
 
-You can build the interface and deploy the HTML page to GitHub Pages,
+You can build the interface and deploy the HTML page to GitHub Pages
+(GitLab pages not yet supported),
 after which it is usable for everyone.
 
 ## Prerequisites
 
-1.  A Text-Fabric dataset that is registered as a TF-App, e.g. `nena` in
-    [github.com/annotation/app-nena](https://github.com/annotation/app-nena).
+1.  A Text-Fabric dataset that has a TF-App, e.g. `CambridgeSemiticsLab/nena_tf`
+    [github.com/CambridgeSemiticsLab/nena_tf](https://github.com/CambridgeSemiticsLab/nena_tf).
     This is the normative example for now.
-1.  Within that app's repo, a subdirectory
-    [layeredsearch](https://github.com/annotation/app-nena/tree/master/layeredsearch)
+1.  An accompanying repository in the same organization, with the same name
+    but with `-search` appended to the name.
+1.  Within that `-search` repo, a subdirectory
+    [layeredsearch](https://github.com/CambridgeSemiticsLab/nena_tf-search/tree/master/layeredsearch)
     with definitions of search interfaces
     (you can define multiple search interfaces for one dataset).
     Within this directory:
@@ -56,21 +59,30 @@ from shutil import copy
 from datetime import datetime as dt
 from subprocess import Popen, PIPE
 from time import sleep
-from zipfile import ZIP_DEFLATED, ZipFile
+from zipfile import ZipFile
 from importlib import util
 
 # from tf.fabric import Fabric
 from tf.app import use
 from tf.fabric import Fabric
-from tf.core.helpers import specFromRanges, rangesFromSet, normpath, abspath, expanduser
+from tf.server.command import getPort
+from tf.core.helpers import specFromRanges, rangesFromSet, normpath, abspath
+from tf.parameters import (
+    REPO,
+    LS,
+    ZIP_OPTIONS,
+    backendRep,
+    URL_TFDOC,
+    APP_CONFIG,
+)
 
 from .gh import deploy
 from .help import HELP
 
-ZIP_OPTIONS = dict(compression=ZIP_DEFLATED, compresslevel=6)
-T_F = "text-fabric"
-LS = "layeredsearch"
-CONFIG_FILE = normpath(f"{os.path.dirname(abspath(__file__))}/config.yaml")
+TEMP = "_temp"
+ZIP_OPTIONS = {"compresslevel": 6, **ZIP_OPTIONS}
+
+CONFIG_FILE = normpath(f"{os.path.dirname(abspath(__file__))}/{APP_CONFIG}")
 STATIC_DIR = normpath(f"{os.path.dirname(os.path.dirname(abspath(__file__)))}/static")
 
 
@@ -94,30 +106,37 @@ def readArgs():
     class Args:
         pass
 
+    Args.backend = None
     Args.dataset = None
+    Args.repo = None
     Args.client = None
     Args.command = None
 
     args = sys.argv[1:]
 
-    if not len(args) or args[0] in {"-h", "--help", "help"}:
+    if not len(args) or any(arg in {"-h", "--help", "help"} for arg in args):
         console(HELP)
-        console("Missing dataset")
         return None
 
+    if not len(args):
+        console("Missing org/repo")
+        return None
+
+    newArgs = []
+    for arg in args:
+        if arg.startswith("--backend="):
+            Args.backend = arg[11:]
+        else:
+            newArgs.append(arg)
+    args = newArgs
+
     dataset = args[0]
-    Args.dataset = dataset
     args = args[1:]
 
     if not len(args):
-        console(HELP)
         console("Missing client or command")
         return None
 
-    if args[0] in {"-h", "--help", "help"}:
-        console(HELP)
-        return None
-
     if args[0] in {"serve", "ship", "make"}:
         client = None
         command = args[0]
@@ -126,15 +145,14 @@ def readArgs():
         client = args[0]
         args = args[1:]
 
-        if not len(args) or args[0] in {"-h", "--help", "help"}:
-            console(HELP)
-            if not len(args):
-                console("No command given")
+        if not len(args):
+            console("No command given")
             return None
 
         command = args[0]
         args = args[1:]
 
+    Args.dataset = dataset
     Args.client = client
     Args.command = command
     Args.folder = None
@@ -166,9 +184,8 @@ def readArgs():
 
     elif command == "make":
         if len(args) < 1:
-            console(HELP)
-            console("Missing output folder argument")
-            return None
+            Args.folder = None
+            Args.appFolder = None
         else:
             Args.folder = args[1] if len(args) > 1 else args[0]
             Args.appFolder = args[0] if len(args) > 1 else None
@@ -184,7 +201,14 @@ def readArgs():
 
 class Make:
     def __init__(
-        self, dataset, client, A=None, folder=None, appFolder=None, debugState=None
+        self,
+        dataset,
+        client,
+        backend=None,
+        A=None,
+        folder=None,
+        appFolder=None,
+        debugState=None,
     ):
         if A is not None:
             self.A = A
@@ -193,6 +217,7 @@ class Make:
             pass
 
         self.C = C
+        self.backend = backend
         self.dataset = dataset
         self.client = client
         self.folder = normpath(folder)
@@ -201,8 +226,10 @@ class Make:
         self.good = True
 
         if dataset:
+            console("configuring ...")
             if not self.config():
                 self.good = False
+            console("success" if self.good else "failed")
 
     def doCommand(self, command):
         if command == "serve":
@@ -232,13 +259,25 @@ class Make:
 
     def config(self):
         C = self.C
+
+        backend = self.backend
         dataset = self.dataset
+        parts = dataset.split("/")
+        if len(parts) != 2:
+            console("Dataset not given as org/repo")
+            return False
+        (org, repo) = parts
+        self.org = org
+        self.repo = repo
+
         client = self.client
         folder = self.folder
         appFolder = self.appFolder
         versionFile = f"{STATIC_DIR}/version.yaml"
         self.versionFile = versionFile
 
+        bUrl = backendRep(backend, "rep")
+
         with open(versionFile) as fh:
             settings = yaml.load(fh, Loader=yaml.FullLoader)
             lsVersion = settings["lsVersion"]
@@ -246,30 +285,29 @@ class Make:
         with open(CONFIG_FILE) as fh:
             mainConfig = yaml.load(fh, Loader=yaml.FullLoader)
 
+        cloneBase = backendRep(backend, "clone")
+
         c = dict(
-            dataset=dataset,
+            backend=backend,
+            org=org,
+            repo=repo,
             client=client,
             lsVersion=lsVersion,
             mainConfig=mainConfig,
-            gh=expanduser("~/github"),
-            ghUrl="https://github.com",
-            nbUrl="https://nbviewer.jupyter.org/github",
-            ghPages="github.io",
-            nbTutUrl="«nbUrl»/annotation/tutorials/tree/master",
-            lsDocUrl=f"https://«org».«ghPages»/{T_F}/tf/about/clientmanual.html",
-            lsDocSimpleUrl=f"https://«org».«ghPages»/{T_F}/tf/about/manual.html",
-            org="annotation",
-            repo="app-«dataset»",
+            lsDocUrl=f"{URL_TFDOC}/about/clientmanual.html",
+            lsDocSimpleUrl=f"{URL_TFDOC}/about/manual.html",
+            searchRepo="«repo»-search",
             rel="site",
-            generatorUrl=f"https://«org».«ghPages»/{T_F}/tf/client/make/build.html",
-            sourceUrl="«ghUrl»/«org»/«repo»/tree/master/layeredsearch",
-            issueUrl="«ghUrl»/«org»/«repo»/issues",
-            tutUrl="«nbTutUrl»/«dataset»/start.ipynb",
+            generatorUrl=f"{URL_TFDOC}/client/make/build.html",
+            sourceUrl=f"{bUrl}/«org»/«searchRepo»/tree/master/layeredsearch",
+            issueUrl=f"{bUrl}/«org»/«searchRepo»/issues",
+            tutUrl=f"{backendRep(backend, 'urlnb')}/«org»/«repo»/blob/master/tutorial/start.ipynb",
             staticDir=STATIC_DIR,
-            appDir="«gh»/«org»/«repo»",
+            appDir=f"{cloneBase}/«org»/«searchRepo»",
+            localDir=f"«appDir»/{TEMP}",
             configDir=f"«appDir»/{LS}" if appFolder is None else f"{appFolder}/{LS}",
-            lsConfig="«configDir»/config.yaml",
-            clientConfigFile="«configDir»/«client»/config.yaml",
+            lsConfig=f"«configDir»/{APP_CONFIG}",
+            clientConfigFile=f"«configDir»/«client»/{APP_CONFIG}",
             clientMake="mkdata",
             clientMakeDir="«configDir»/«client»",
             clientMakeFile="«clientMakeDir»/«clientMake».py",
@@ -284,6 +322,7 @@ class Make:
             index="«htmlInDir»/index.html",
             siteDir="«appDir»/«rel»" if folder is None else folder,
             appClientDir="«siteDir»/«client»",
+            textClientDir="«localDir»/«client»",
             pngOutDir="«appClientDir»/png",
             cssOutDir="«appClientDir»/css",
             htmlOutDir="«siteDir»",
@@ -333,7 +372,7 @@ class Make:
 
         lsConfig = c["lsConfig"]
         if not os.path.exists(lsConfig):
-            console(f"No config.yaml found for {dataset}: {lsConfig}")
+            console(f"No {APP_CONFIG} found for {org}/{repo}: {lsConfig}")
             return None
 
         with open(lsConfig) as fh:
@@ -346,7 +385,7 @@ class Make:
             clientConfigFile = c["clientConfigFile"]
             if not os.path.exists(clientConfigFile):
                 console(
-                    f"No config.yaml found for {dataset}:{client}: {clientConfigFile}"
+                    f"No {APP_CONFIG} found for {org}/{repo}:{client}: {clientConfigFile}"
                 )
                 return None
 
@@ -359,9 +398,9 @@ class Make:
             self.importMake(c=c)
 
             d = dict(
-                dataLocation="«gh»/«data.org»/«data.repo»/«data.rel»",
-                dataUrl="«ghUrl»/«data.org»/«data.repo»/tree/master/«data.rel»/«data.version»",
-                writingUrl="https://«org».«ghPages»/text-fabric/tf/writing/«writing».html",
+                dataLocation=f"{cloneBase}/«data.org»/«data.repo»/«data.rel»",
+                dataUrl=f"{bUrl}/«data.org»/«data.repo»/tree/master/«data.rel»/«data.version»",
+                writingUrl=f"{URL_TFDOC}/writing/«writing».html",
                 urls=dict(
                     cheatsheet=(
                         "regexp cheatsheet",
@@ -378,8 +417,8 @@ class Make:
                     ),
                     maker=(
                         None,
-                        "https://dans.knaw.nl/en/front-page?set_language=en",
-                        "Website of DANS = Data Archiving and Networked Services",
+                        "https://huc.knaw.nl/di/text/",
+                        "Website of KNAW/Humanities Cluster/Team Text",
                     ),
                     corpus=(
                         None,
@@ -408,7 +447,7 @@ class Make:
                     ),
                     tf=(
                         None,
-                        "https://«org».«ghPages»/text-fabric/tf/",
+                        URL_TFDOC,
                         "Text-Fabric documentation website",
                     ),
                     lsdoc=(
@@ -432,12 +471,12 @@ class Make:
                         "online repository of the underlying text-fabric data",
                     ),
                     generator=(
-                        f"{T_F}/client",
+                        f"{REPO}/client",
                         "«generatorUrl»",
                         "the generator of this search interface",
                     ),
                     source=(
-                        "«repo»",
+                        "«searchRepo»",
                         "«sourceUrl»",
                         "source code of the definition of this search interface",
                     ),
@@ -457,7 +496,7 @@ class Make:
                         "characters and transliteration for «writing»",
                     ),
                     related=(
-                        "text-fabric «dataset»",
+                        "text-fabric «org»/«repo»",
                         "«tutUrl»",
                         "using Text-Fabric on the same corpus",
                     ),
@@ -495,7 +534,8 @@ class Make:
 
     def importMake(self, c=None):
         client = self.client
-        dataset = self.dataset
+        org = self.org
+        repo = self.repo
 
         if c is None:
             C = self.C
@@ -508,7 +548,7 @@ class Make:
             clientMakeFile = c["clientMakeFile"]
 
         try:
-            moduleName = f"tf.client.ls.{dataset}.{client}.{clientMake}"
+            moduleName = f"tf.client.ls.{org}.{repo}.{client}.{clientMake}"
             spec = util.spec_from_file_location(moduleName, clientMakeFile)
             code = util.module_from_spec(spec)
             sys.path.insert(0, clientMakeDir)
@@ -518,7 +558,7 @@ class Make:
             self.record = types.MethodType(code.record, self)
 
         except Exception as e:
-            console(f"Cannot make data for {dataset}:{client}: {str(e)}")
+            console(f"Cannot make data for {org}/{repo}:{client}: {str(e)}")
             return None
 
     def makeClientSettings(self):
@@ -544,8 +584,8 @@ class Make:
             mainConfig=C.mainConfig,
             defs=dict(
                 lsVersion=C.lsVersion,
-                dataset=C.dataset,
                 client=C.client,
+                backend=C.backend,
                 org=C.org,
                 repo=C.repo,
                 urls=C.urls,
@@ -616,16 +656,21 @@ class Make:
                 continue
 
             visible[level] = {layer: ti[layer].get("visible", False) for layer in ti}
-            layers[level] = {
-                layer: dict(
-                    valueMap=invertMap(ti[layer].get("legend", None)),
+            layers[level] = {}
+
+            for layer in ti:
+                explain = ti[layer].get("explain", None)
+                valueMap = invertMap(ti[layer].get("legend", None))
+                valueMapAcro = type(valueMap) is dict
+                layers[level][layer] = dict(
+                    explain=explain,
+                    valueMap=valueMap,
+                    valueMapAcro=valueMapAcro,
                     tip=ti[layer].get("tip", False),
                     pos=ti[layer]["pos"] or layer,
                     pattern=ti[layer].get("example", ""),
                     description=ti[layer].get("description", ""),
                 )
-                for layer in ti
-            }
 
         for (k, v) in (
             ("ntypesinit", ntypesinit),
@@ -641,9 +686,13 @@ class Make:
 
     def loadTf(self):
         C = self.C
-        dataset = C.dataset
+        backend = C.backend
+        org = C.org
+        repo = C.repo
         version = C.data["version"]
-        A = use(f"{dataset}:clone", checkout="clone", version=version)
+        A = use(
+            f"{org}/{repo}:clone", checkout="clone", backend=backend, version=version
+        )
         self.A = A
 
     def makeConfig(self):
@@ -726,6 +775,7 @@ class Make:
         A = self.A
         layerSettings = C.layerSettings
         memSavingMethod = C.memSavingMethod
+        debug = self.debugState == "on"
 
         up = self.up
         recorders = self.recorders
@@ -782,8 +832,15 @@ class Make:
         if not os.path.exists(destData):
             os.makedirs(destData, exist_ok=True)
 
+        if debug:
+            textLoc = C.textClientDir
+            if not os.path.exists(textLoc):
+                os.makedirs(textLoc, exist_ok=True)
+
         def writeDataFile(name, address, thisData, asString=False):
-            path = f"{destData}/{name.lower()}.js"
+            parent = textLoc if debug and asString else destData
+            file = name.lower() + (".txt" if debug and asString else ".js")
+            path = f"{parent}/{file}"
             heading = f"corpusData[{address}] = "
             with open(path, "w") as fh:
                 fh.write(heading)
@@ -799,7 +856,10 @@ class Make:
                         indent=None,
                         separators=(",", ":"),
                     )
-                A.info(f"Data {name} stored in {path}")
+                if asString:
+                    A.info(f"Data {name} also written to {path}")
+                else:
+                    A.info(f"Data {name} stored in {path}")
 
         init = ["var corpusData = {}\n"]
 
@@ -814,6 +874,15 @@ class Make:
                             f'"{partName}"]["{nType}"]["{layer}"',
                             lrData,
                         )
+                if debug and partName == "texts":
+                    for (nType, tpData) in partData.items():
+                        for (layer, lrData) in tpData.items():
+                            writeDataFile(
+                                f"{partName}-{nType}-{layer}",
+                                f'"{partName}"]["{nType}"]["{layer}"',
+                                lrData,
+                                asString=True,
+                            )
             else:
                 writeDataFile(partName, f'"{partName}"', partData)
         with open(C.jsInit, "w") as fh:
@@ -830,7 +899,7 @@ class Make:
         whiteRe = re.compile(r"""^\s+$""", re.M)
         nlRe = re.compile(r"""\n\n+""")
 
-        def getModule(module):
+        def getJSModule(module):
             with open(f"{C.jsOutDir}/{module}") as fh:
                 text = fh.read()
             text = importRe.sub("", text)
@@ -854,7 +923,7 @@ class Make:
                 modules.append(entry.name)
         console(", ".join(module[0:-3] for module in modules))
 
-        content = {module: getModule(module) for module in modules}
+        content = {module: getJSModule(module) for module in modules}
 
         header = """\
 /*eslint-env jquery*/
@@ -887,7 +956,7 @@ class Make:
         clients = {}
 
         for thisClient in self.getAllClients():
-            thisConfig = f"{C.configDir}/{thisClient}/config.yaml"
+            thisConfig = f"{C.configDir}/{thisClient}/{APP_CONFIG}"
             if os.path.exists(thisConfig):
                 with open(thisConfig) as fh:
                     desc = yaml.load(fh, Loader=yaml.FullLoader).get("short", "")
@@ -897,7 +966,7 @@ class Make:
 
         with open(C.index) as fh:
             template = fh.read()
-            htmlIndex = template.replace("«dataset»", C.dataset)
+            htmlIndex = template.replace("«org»", C.org).replace("«repo»", C.repo)
             htmlIndex = htmlIndex.replace("«client»", C.client)
 
             html = []
@@ -934,12 +1003,12 @@ class Make:
             )
             htmlNormal = htmlNormal.replace("«corpus»", corpusScripts)
             htmlNormal = htmlNormal.replace("«v»", f"?v={lsVersion}")
-            htmlNormal = htmlNormal.replace("«dataset»", C.dataset)
+            htmlNormal = htmlNormal.replace("«org»", C.org).replace("«repo»", C.repo)
             htmlNormal = htmlNormal.replace("«client»", C.client)
             htmlLocal = template.replace("«js»", f'''defer src="{C.jsAll}"''')
             htmlLocal = htmlLocal.replace("«corpus»", corpusScripts)
             htmlLocal = htmlLocal.replace("«v»", "")
-            htmlLocal = htmlLocal.replace("«dataset»", C.dataset)
+            htmlLocal = htmlLocal.replace("«org»", C.org).replace("«repo»", C.repo)
             htmlLocal = htmlLocal.replace("«client»", C.client)
 
         with open(C.htmlClient, "w") as fh:
@@ -975,6 +1044,8 @@ class Make:
 
         However, if the debugState is on, we skip all steps that are unneccesary
         to see the updated client working.
+        But we do save an extra copy of the texts to the local dir in such a way
+        that they can be easily inspected.
         """
 
         # copy over the static files
@@ -1063,12 +1134,23 @@ class Make:
         C = self.C
         appDir = C.appDir
         siteDir = C.siteDir
-        dataset = self.dataset
+        backend = self.backend
+        org = self.org
+        repo = self.repo
         client = self.client
         clients = self.getAllClients() if allClients or client is None else [client]
-        console(f"Publishing {dataset}:{','.join(clients)} from {siteDir} ...")
+        domain = backendRep(backend, "pages")
+        console(
+            f"Publishing on {domain} "
+            f"{org}/{repo}:{','.join(clients)} from {siteDir} ..."
+        )
         os.chdir(appDir)
-        deploy(C.org, C.repo)
+        if backend is None:
+            deploy(C.org, C.searchRepo)
+        else:
+            console(
+                f"Deploying Pages on GitLab ({backend}) not yet supported", error=True
+            )
 
     def ship(self, publish=True):
         self.adjustVersion()
@@ -1090,13 +1172,20 @@ class Make:
     def serve(self):
         C = self.C
         os.chdir(C.siteDir)
+        port = getPort(portBase=8000)
+        if port is None:
+            print("Cannot find a free port between 8000 and 8100")
+            return
 
-        console(f"HTTP serving files in {C.siteDir}")
+        console(f"HTTP serving files in {C.siteDir} on port {port}")
         server = Popen(
-            ["python3", "-m", "http.server"], stdout=PIPE, bufsize=1, encoding="utf-8"
+            ["python3", "-m", "http.server", str(port)],
+            stdout=PIPE,
+            bufsize=1,
+            encoding="utf-8",
         )
         sleep(1)
-        webbrowser.open("http://localhost:8000/index.html", new=2, autoraise=True)
+        webbrowser.open(f"http://localhost:{port}/index.html", new=2, autoraise=True)
         stopped = server.poll()
         if not stopped:
             try:
@@ -1211,10 +1300,17 @@ class Make:
         return clients
 
 
-def makeSearchClients(dataset, folder, appFolder, dataDir=None):
+def makeSearchClients(dataset, folder, appFolder, backend=None, dataDir=None):
     DEBUG_STATE = "off"
 
-    Mk = Make(dataset, None, folder=folder, appFolder=appFolder, debugState=DEBUG_STATE)
+    Mk = Make(
+        dataset,
+        None,
+        backend=backend,
+        folder=folder,
+        appFolder=appFolder,
+        debugState=DEBUG_STATE,
+    )
     clients = Mk.getAllClients()
     # version = Mk.C.data["version"]
 
@@ -1230,6 +1326,7 @@ def makeSearchClients(dataset, folder, appFolder, dataDir=None):
         ThisMk = Make(
             dataset,
             client,
+            backend=backend,
             A=A,
             folder=folder,
             appFolder=appFolder,
@@ -1244,6 +1341,7 @@ def main():
     if Args is None:
         return 0
 
+    backend = Args.backend
     dataset = Args.dataset
     client = Args.client
     command = Args.command
@@ -1259,8 +1357,9 @@ def main():
             return
 
     Mk = Make(
-        Args.dataset,
-        Args.client,
+        dataset,
+        client,
+        backend=backend,
         folder=folder,
         appFolder=appFolder,
         debugState=debugState,
@@ -1272,6 +1371,7 @@ def main():
             ThisMk = Make(
                 dataset,
                 client,
+                backend=backend,
                 folder=folder,
                 appFolder=appFolder,
                 debugState=debugState,
