@@ -50,7 +50,7 @@ good = cv.walk(
     featureMeta=featureMeta,
     warn=True,
     force=False,
-    silent=False,
+    silent="auto",
 )
 
 if good:
@@ -117,12 +117,14 @@ that steer the graph building:
 *   `tf.convert.walker.CV.node`
 *   `tf.convert.walker.CV.terminate`
 *   `tf.convert.walker.CV.resume`
+*   `tf.convert.walker.CV.link`
+*   `tf.convert.walker.CV.linked`
 *   `tf.convert.walker.CV.feature`
 *   `tf.convert.walker.CV.edge`
 *   `tf.convert.walker.CV.meta`
 *   `tf.convert.walker.CV.occurs`
-*   `tf.convert.walker.CV.linked`
 *   `tf.convert.walker.CV.active`
+*   `tf.convert.walker.CV.activeNodes`
 *   `tf.convert.walker.CV.activeTypes`
 *   `tf.convert.walker.CV.get` and `cv.get(feature, nf, nt)`
 *   `tf.convert.walker.CV.stop`
@@ -141,6 +143,7 @@ import re
 
 from ..parameters import WARP, OTYPE, OSLOTS, OTEXT
 from ..core.helpers import itemize, isInt
+from ..core.timestamp import SILENT_D, silentConvert
 
 
 class CV(object):
@@ -151,12 +154,19 @@ class CV(object):
     F = "feature"
     E = "edge"
 
-    def __init__(self, TF, silent=False):
+    def __init__(self, TF, silent=SILENT_D):
+        """The object that contains the walker conversion machinery.
+
+        silent: string, optional `tf.core.timestamp.SILENT_D`
+            See `tf.core.timestamp.Timestamp`
+        """
+
         self.TF = TF
         tmObj = TF.tmObj
         isSilent = tmObj.isSilent
         setSilent = tmObj.setSilent
 
+        silent = silentConvert(silent)
         self.wasSilent = isSilent()
         setSilent(silent)
 
@@ -290,9 +300,6 @@ class CV(object):
 
             `None` suppress all warnings.
 
-        silent: boolean, optional `None`
-            By this you can suppress informational messages: `silent=True`.
-
         force: boolean, optional `False`
             This forces the process to continue after errors.
             Your TF set might not be valid.
@@ -314,6 +321,11 @@ class CV(object):
             to TF.
 
             See `tf.convert.walker` for more details.
+
+        Returns
+        -------
+        boolean
+            Whether the operation was successful
         """
 
         tmObj = self.TF.tmObj
@@ -441,6 +453,11 @@ class CV(object):
                 )
             self.sectionFeatures = sectionInfo["sectionFeatures"]
             self.sectionTypes = sectionInfo["sectionTypes"]
+            self.featFromSectionType = {
+                typ: feat
+                for (typ, feat) in zip(self.sectionTypes, self.sectionFeatures)
+            }
+            self.sectionSet = set(self.sectionTypes)
 
             structureInfo = {}
             for f in ("structureTypes", "structureFeatures"):
@@ -465,7 +482,7 @@ class CV(object):
                 self.structureTypes = []
             self.structureFeatures = structureInfo["structureFeatures"]
             self.structureTypes = structureInfo["structureTypes"]
-            self.featFromType = {
+            self.featFromStructureType = {
                 typ: feat
                 for (typ, feat) in zip(self.structureTypes, self.structureFeatures)
             }
@@ -716,7 +733,9 @@ class CV(object):
 
             for s in slots:
                 if not 1 <= s <= maxSlot:
-                    errors[f'slot out of range in `cv.node(({nType}, {seq}))`'].append(f"{s}")
+                    errors[f"slot out of range in `cv.node(({nType}, {seq}))`"].append(
+                        f"{s}"
+                    )
                 else:
                     oslots = self.oslots
                     oslots[node].add(s)
@@ -784,6 +803,68 @@ class CV(object):
             self._checkSecLevel(node, before=None)
             curEmbedders.add(node)
 
+    def link(self, node, slots):
+        """Link the given, existing slots to a node.
+
+            cv.link(n)
+
+        Sometimes the automatic linking of slots to nodes is not sufficient.
+
+        This happens when you feel the need to construct a node retro-actively,
+        when the slots that need to be linked to it have already been created.
+
+        This action is precisely meant for that.
+
+        Parameters
+        ----------
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+        slots: iterable of integer
+
+        Returns
+        -------
+        boolean
+        """
+
+        oslots = self.oslots
+
+        good = True
+
+        for seq in slots:
+            oslots[node].add(seq)
+
+        return good
+
+    def linked(self, node):
+        """Returns the slots `ss` to which a node is currently linked.
+
+            ss = cv.linked(n)
+
+
+        If you construct non-slot nodes without linking them to slots,
+        they will be removed when TF validates the collective result
+        of the action methods.
+
+        If you want to prevent that, you can insert an extra slot, but in order
+        to do so, you have to detect that a node is still unlinked.
+
+        This action is precisely meant for that.
+
+        Parameters
+        ----------
+        node: tuple
+            A node reference, obtained by one of the actions `slot` or `node`.
+
+        Returns
+        -------
+        tuple of integer
+
+        The slots are returned as a tuple of integers.
+        """
+
+        oslots = self.oslots
+        return tuple(oslots.get(node, []))
+
     def feature(self, node, **features):
         """Add **node features**.
 
@@ -936,34 +1017,6 @@ class CV(object):
 
         self.good = self._checkFeatMeta(feat, featMeta) and good and self.good
 
-    def linked(self, node):
-        """Returns the slots `ss` to which a node is currently linked.
-
-            ss = cv.linked(n)
-
-
-        If you construct non-slot nodes without linking them to slots,
-        they will be removed when TF validates the collective result
-        of the action methods.
-
-        If you want to prevent that, you can insert an extra slot, but in order
-        to do so, you have to detect that a node is still unlinked.
-
-        This action is precisely meant for that.
-
-        Parameters
-        ----------
-        node: tuple
-            A node reference, obtained by one of the actions `slot` or `node`.
-
-        Returns
-        -------
-        boolean
-        """
-
-        oslots = self.oslots
-        return tuple(oslots.get(node, []))
-
     def active(self, node):
         """Returns whether a node is currently active.
 
@@ -990,6 +1043,29 @@ class CV(object):
 
         return node in self.curEmbedders
 
+    def activeNodes(self, nTypes=None):
+        """The currently active nodes, i.e. the embedders.
+
+            nodes = cv.activeNodes()
+            nodes = cv.activeNodes(nTypes=("sentence", "clause"))
+
+        Parameters
+        ----------
+        nTypes: iterable optional `None`
+            If None, all active nodes are returned.
+            Else the iterable lists a few node types,
+            and only active nodes in these types are returned.
+
+        Returns
+        -------
+        set
+        """
+
+        if nTypes is None:
+            return set(self.curEmbedders)
+        nTypes = set(nTypes)
+        return {n for n in self.curEmbedders if n[0] in nTypes}
+
     def activeTypes(self):
         """The node types of the currently active nodes, i.e. the embedders.
 
@@ -1149,7 +1225,8 @@ class CV(object):
             for (nType, seq) in curEmbedders:
                 embedCount[nType] += 1
             for (nType, amount) in sorted(
-                embedCount.items(), key=lambda x: (-x[1], x[0]),
+                embedCount.items(),
+                key=lambda x: (-x[1], x[0]),
             ):
                 errors["Unterminated nodes"].append(f"{nType}: {amount} x")
 
@@ -1247,7 +1324,7 @@ class CV(object):
         if not self.good and not self.force:
             return
 
-        info("checking features ... ")
+        info("checking (section) features ... ")
 
         intFeatures = self.intFeatures
         metaData = self.metaData
@@ -1265,17 +1342,20 @@ class CV(object):
                 and feat not in edgeFeatures
             ):
                 errors["intFeatures"].append(
-                    f'"{feat}" is declared as integer valued, but this feature does not occur'
+                    f'"{feat}" is declared as integer valued, '
+                    "but this feature does not occur"
                 )
         for nType in self.sectionTypes:
             if nType not in nodes:
                 errors["sections"].append(
-                    f'node type "{nType}" is declared as a section type, but this node type does not occur'
+                    f'node type "{nType}" is declared as a section type, '
+                    "but this node type does not occur"
                 )
         for feat in self.sectionFeatures:
             if feat not in nodeFeatures:
                 errors["sections"].append(
-                    f'"{feat}" is declared as a section feature, but this node feature does not occur'
+                    f'"{feat}" is declared as a section feature, '
+                    "but this node feature does not occur"
                 )
         for nType in self.structureTypes:
             if nType not in nodes:
@@ -1286,26 +1366,39 @@ class CV(object):
         for feat in self.structureFeatures:
             if feat not in nodeFeatures:
                 errors["structure"].append(
-                    f'"{feat}" is declared as a structure feature, but this node feature does not occur'
+                    f'"{feat}" is declared as a structure feature, '
+                    "but this node feature does not occur"
                 )
                 nodeFeatures[feat] = {}
 
+        sectionSet = self.sectionSet
         structureSet = self.structureSet
-        featFromType = self.featFromType
+        featFromSectionType = self.featFromSectionType
+        featFromStructureType = self.featFromStructureType
+
         for nType in nodes:
-            if nType not in structureSet:
-                continue
-            feat = featFromType[nType]
-            for seq in nodes[nType]:
-                if (nType, seq) not in nodeFeatures[feat]:
-                    errors["structure features"].append(
-                        f'"structure element "{nType}" {seq} has no value for "{feat}"'
-                    )
+            if nType in structureSet:
+                feat = featFromStructureType[nType]
+                for seq in nodes[nType]:
+                    if (nType, seq) not in nodeFeatures[feat]:
+                        errors["structure features"].append(
+                            f'"structure element "{nType}" {seq} '
+                            f'has no value for "{feat}"'
+                        )
+            if nType in sectionSet:
+                feat = featFromSectionType[nType]
+                for seq in nodes[nType]:
+                    if (nType, seq) not in nodeFeatures[feat]:
+                        errors["section features"].append(
+                            f'"section element "{nType}" {seq} '
+                            f'has no value for "{feat}"'
+                        )
 
         for feat in self.textFeatures:
             if feat not in nodeFeatures:
                 errors["text formats"].append(
-                    f'"{feat}" is used in a text format, but this node feature does not occur'
+                    f'"{feat}" is used in a text format, '
+                    "but this node feature does not occur"
                 )
 
         for feat in WARP:
