@@ -4,6 +4,8 @@
 It provides methods to navigate nodes and edges and lookup features.
 """
 
+from textwrap import wrap
+
 from .helpers import flattenToSet, console, fitemize, unexpanduser as ux
 from .nodes import Nodes
 from .locality import Locality
@@ -12,6 +14,7 @@ from .edgefeature import EdgeFeatures
 from .computed import Computeds
 from .text import Text
 from ..search.search import Search
+from ..core.timestamp import SILENT_D, DEEP, silentConvert
 
 API_REFS = dict(
     AllComputeds=("Computed", "computedall", "computed-data"),
@@ -178,7 +181,7 @@ class Api(object):
             Only relevant if `pretty=True`: controls what metadata of the feature
             should be printed.
 
-            If it is None, False, or the empty string ir empty list:
+            If it is None, False, or the empty string or empty list:
             no metadata will be printed.
 
             It it is the boolean value True: all metadata will be printed.
@@ -263,6 +266,7 @@ class Api(object):
                     kind = f" {kind:<10}"
                     fSource = ""
                     metaRep = ""
+                    heading = f"{fName:<20}{kind}{fSource}"
                 else:
                     fKind = fInfo["kind"]
                     fMeta = fInfo.get("meta", {})
@@ -292,16 +296,22 @@ class Api(object):
                         metaKeys = fitemize(meta)
                         metaInfo = {k: fMeta[k] for k in metaKeys if k in fMeta}
 
+                    heading = f"{fName:<20}{kind}{fSource}"
                     metaRep = ""
+                    indent = " " * (len(heading) + 1)
                     if metaInfo:
                         if len(metaKeys) == 1:
-                            metaRep = metaInfo.get(metaKeys[0], "")
-                            metaRep = f" {metaRep}" if metaRep else ""
+                            value = metaInfo.get(metaKeys[0], "")
+                            value = "\n".join(wrap(value, width=80, subsequent_indent=indent))
+                            metaRep = f" {value}" if value else ""
                         else:
+                            indent = " " * 21
                             for k in metaKeys:
-                                metaRep += f"\n\t{k:<20} = {metaInfo.get(k, '')}"
+                                value = metaInfo.get(k, "")
+                                value = "\n".join(wrap(value, width=80, subsequent_indent=f"\t{indent}  "))
+                                metaRep += f"\n\t{k:<20} = {value}"
 
-                msg = f"{fName:<20}{kind}{fSource}{metaRep}"
+                msg = f"{heading}{metaRep}"
                 print(msg)
             return None
         return info
@@ -420,7 +430,7 @@ class Api(object):
             TF.load(
                 needToLoad,
                 add=True,
-                silent="deep",
+                silent=DEEP,
             )
             loadedFeatures |= needToLoad
         return loadedFeatures
@@ -448,6 +458,7 @@ def addText(api):
     api.Text = api.T
 
 
-def addSearch(api, silent):
+def addSearch(api, silent=SILENT_D):
+    silent = silentConvert(silent)
     api.S = Search(api, silent)
     api.Search = api.S
