@@ -1,8 +1,10 @@
 import os
+from array import array
+import gc
 import pickle
+from pickletools import optimize
 import gzip
 import collections
-from array import array
 import time
 from datetime import datetime
 from ..parameters import PACK_VERSION, PICKLE_PROTOCOL, GZIP_LEVEL, OTYPE, OSLOTS, OTEXT
@@ -15,6 +17,7 @@ from .helpers import (
     check32,
     console,
 )
+from .timestamp import SILENT_D, silentConvert
 
 ERROR_CUTOFF = 20
 
@@ -61,22 +64,31 @@ class Data(object):
         self.dataError = False
         self.dataType = "str"
 
-    def load(self, metaOnly=False, silent=None):
+    def load(self, metaOnly=False, silent=SILENT_D, _withGc=True):
+        """Load a feature.
+
+        _withGc: boolean, optional True
+            If False, it disables the Python garbage collector before
+            loading features. Used to experiment with performance.
+        """
+
+        silent = silentConvert(silent)
         tmObj = self.tmObj
         isSilent = tmObj.isSilent
         setSilent = tmObj.setSilent
         indent = tmObj.indent
-        warning = tmObj.warning
+        info = tmObj.info
         error = tmObj.error
 
-        if silent is not None:
-            wasSilent = isSilent()
-            setSilent(silent)
-        indent(level=1, reset=True)
+        wasSilent = isSilent()
+        setSilent(silent)
+        indent(level=True, reset=True)
         origTime = self._getModified()
         binTime = self._getModified(bin=True)
         sourceRep = (
-            ", ".join(dep.fileName for dep in self.dependencies)
+            ", ".join(
+                dep.fileName for dep in self.dependencies if isinstance(dep, Data)
+            )
             if self.method
             else self.dirName
         )
@@ -104,7 +116,7 @@ class Data(object):
             try:
                 if not origTime:
                     actionRep = "b"
-                    good = self._readDataBin()
+                    good = self._readDataBin(_withGc=_withGc)
                 elif not binTime or origTime > binTime:
                     actionRep = "C" if self.method else "T"
                     good = (
@@ -124,7 +136,7 @@ class Data(object):
                         if self.isConfig or metaOnly:
                             actionRep = "M"
                         else:
-                            good = self._readDataBin()
+                            good = self._readDataBin(_withGc=_withGc)
             except MemoryError:
                 console(MEM_MSG)
                 good = False
@@ -135,7 +147,7 @@ class Data(object):
                 actionRep == "M" or (actionRep == "B" and self.method)
             ):
                 pass
-            warning(
+            info(
                 msgFormat.format(actionRep, self.fileName, sourceRep),
                 cache=1 if actionRep in "CT" else -1,
             )
@@ -143,25 +155,24 @@ class Data(object):
             self.dataError = True
             error(msgFormat.format(actionRep, self.fileName, sourceRep))
 
-        if silent is not None:
-            setSilent(wasSilent)
+        setSilent(wasSilent)
+        indent(level=False)
         return good
 
     def unload(self):
         self.data = None
         self.dataLoaded = False
 
-    def save(self, overwrite=False, nodeRanges=False, silent=None):
+    def save(self, overwrite=False, nodeRanges=False, silent=SILENT_D):
+        silent = silentConvert(silent)
         tmObj = self.tmObj
         isSilent = tmObj.isSilent
         setSilent = tmObj.setSilent
 
-        if silent is not None:
-            wasSilent = isSilent()
-            setSilent(silent)
+        wasSilent = isSilent()
+        setSilent(silent)
         result = self._writeTf(overwrite=overwrite, nodeRanges=nodeRanges)
-        if silent is not None:
-            setSilent(wasSilent)
+        setSilent(wasSilent)
         return result
 
     def _setDataType(self):
@@ -170,24 +181,27 @@ class Data(object):
 
         tmObj = self.tmObj
         error = tmObj.error
+        fileName = self.fileName
 
         dataTypesStr = ", ".join(DATA_TYPES)
         if "valueType" in self.metaData:
             dataType = self.metaData["valueType"]
             if dataType not in DATA_TYPES:
                 error(
-                    f'Unknown @valueType: "{dataType}". Should be one of {dataTypesStr}'
+                    f"{fileName}: Unknown @valueType: {dataType}. "
+                    f"Should be one of {dataTypesStr}"
                 )
                 self.dataType = DATA_TYPES[0]
             else:
                 self.dataType = dataType
         else:
-            error(f"Missing @valueType. Should be one of {dataTypesStr}")
+            error(f"{fileName}: Missing @valueType. Should be one of {dataTypesStr}")
             self.dataType = DATA_TYPES[0]
 
     def _readTf(self, metaOnly=False):
         tmObj = self.tmObj
         error = tmObj.error
+        fileName = self.fileName
 
         path = self.path
         if not os.path.exists(path):
@@ -208,7 +222,7 @@ class Data(object):
                 elif text == "@config":
                     self.isConfig = True
                 else:
-                    error(f"Line {i}: missing @node/@edge/@config")
+                    error(f"{fileName}: Line {i}: missing @node/@edge/@config")
                     fh.close()
                     return False
                 continue
@@ -222,7 +236,7 @@ class Data(object):
                 continue
             else:
                 if text != "":
-                    error(f"Line {i}: missing blank line after metadata")
+                    error(f"{fileName}: Line {i}: missing blank line after metadata")
                     fh.close()
                     return False
                 else:
@@ -237,6 +251,7 @@ class Data(object):
     def _readDataTf(self, fh, firstI):
         tmObj = self.tmObj
         error = tmObj.error
+        fileName = self.fileName
 
         errors = collections.defaultdict(list)
         i = firstI
@@ -325,8 +340,10 @@ class Data(object):
         for kind in errors:
             lnk = len(errors[kind])
             error(
-                "{} in lines {}".format(
-                    kind, ",".join(str(ln) for ln in errors[kind][0:ERROR_CUTOFF])
+                "{}: {} in lines {}".format(
+                    fileName,
+                    kind,
+                    ",".join(str(ln) for ln in errors[kind][0:ERROR_CUTOFF]),
                 )
             )
             if lnk > ERROR_CUTOFF:
@@ -365,6 +382,7 @@ class Data(object):
                 oslots = []
                 for n in nodeList:
                     oslots.append(array("I", sorted(data[n])))
+                    # oslots.append(tuple(sorted(data[n])))
                 self.data = (tuple(oslots), maxSlot, maxNode)
             elif isEdge:
                 seen = {}
@@ -396,18 +414,19 @@ class Data(object):
         return not errors
 
     def _compute(self, metaOnly=False):
+        tmObj = self.tmObj
+        isSilent = tmObj.isSilent
         if metaOnly:
             return True
 
         good = True
         for feature in self.dependencies:
-            if not feature.load():
-                good = False
+            if isinstance(feature, Data):
+                if not feature.load(silent=isSilent()):
+                    good = False
         if not good:
             return False
 
-        tmObj = self.tmObj
-
         def info(msg, tm=True):
             tmObj.info(cmpFormat.format(msg), tm=tm, cache=-1)
 
@@ -421,7 +440,9 @@ class Data(object):
             info,
             error,
             *[
-                dep.metaData if dep.fileName == OTEXT else dep.data
+                (dep.metaData if dep.fileName == OTEXT else dep.data)
+                if isinstance(dep, Data)
+                else dep
                 for dep in self.dependencies
             ],
         )
@@ -500,6 +521,7 @@ class Data(object):
     def _writeDataTf(self, fh, nodeRanges=False):
         tmObj = self.tmObj
         error = tmObj.error
+        fileName = self.fileName
 
         data = self.data
         if type(data) is tuple:
@@ -507,7 +529,7 @@ class Data(object):
             # in case it has been loaded from a binary representation
             fName = self.fileName
             if fName not in {OTYPE, OSLOTS}:
-                error("Data type tuple not suitable for non-WARP feature")
+                error(f"{fileName}: Data type tuple not suitable for non-WARP feature")
                 return False
             maxSlot = data[2] if fName == OTYPE else data[1]
             slotType = data[1] if fName == OTYPE else None
@@ -536,7 +558,9 @@ class Data(object):
                         if tfValue is None:
                             fh.write(
                                 "{}{}{}\n".format(
-                                    nodeSpec, "\t" if nodeSpec else "", nodeSpec2,
+                                    nodeSpec,
+                                    "\t" if nodeSpec else "",
+                                    nodeSpec2,
                                 )
                             )
                         else:
@@ -573,33 +597,47 @@ class Data(object):
                     if tfValue is not None:
                         fh.write(
                             "{}{}{}\n".format(
-                                nodeSpec, "\t" if nodeSpec else "", tfValue,
+                                nodeSpec,
+                                "\t" if nodeSpec else "",
+                                tfValue,
                             )
                         )
             else:
                 implicitNode = 1
                 for n in sorted(data):
                     nodeSpec = "" if n == implicitNode else n
-                    implicitNode = n + 1
                     value = data[n]
                     tfValue = value if value is None else tfFromValue(value)
                     if tfValue is not None:
+                        implicitNode = n + 1
                         fh.write(
                             "{}{}{}\n".format(
-                                nodeSpec, "\t" if nodeSpec else "", tfValue,
+                                nodeSpec,
+                                "\t" if nodeSpec else "",
+                                tfValue,
                             )
                         )
         return True
 
-    def _readDataBin(self):
+    def _readDataBin(self, _withGc=True):
+        """Read binary feature data.
+        _withGc: boolean, optional True
+            If False, it disables the Python garbage collector before
+            loading features. Used to experiment with performance.
+        """
+
         tmObj = self.tmObj
         error = tmObj.error
 
         if not os.path.exists(self.binPath):
             error(f'TF reading: feature file "{self.binPath}" does not exist')
             return False
+        if not _withGc:
+            gc.disable()
         with gzip.open(self.binPath, "rb") as f:
             self.data = pickle.load(f)
+        if not _withGc:
+            gc.enable()
         self.dataLoaded = time.time()
         return True
 
@@ -622,7 +660,8 @@ class Data(object):
             return False
         try:
             with gzip.open(self.binPath, "wb", compresslevel=GZIP_LEVEL) as f:
-                pickle.dump(self.data, f, protocol=PICKLE_PROTOCOL)
+                # pickle.dump(self.data, f, protocol=PICKLE_PROTOCOL)
+                f.write(optimize(pickle.dumps(self.data, protocol=PICKLE_PROTOCOL)))
         except Exception as e:
             error(f'Cannot write to file "{self.binPath}" because: {str(e)}')
             self.cleanDataBin()
@@ -637,7 +676,11 @@ class Data(object):
             )
         else:
             if self.method:
-                depsInfo = [dep._getModified() for dep in self.dependencies]
+                depsInfo = [
+                    dep._getModified()
+                    for dep in self.dependencies
+                    if isinstance(dep, Data)
+                ]
                 depsModifieds = [d for d in depsInfo if d is not None]
                 depsModified = None if len(depsModifieds) == 0 else max(depsModifieds)
                 if depsModified is not None:
