@@ -17,6 +17,8 @@ By leaving out volume support, volume support can use FabricCore instead of Fabr
 import os
 
 import collections
+from itertools import chain
+
 from ..parameters import VERSION, NAME, APIREF, LOCATIONS, OTYPE, OSLOTS, OTEXT
 from .data import Data, MEM_MSG
 from .helpers import (
@@ -32,7 +34,7 @@ from .helpers import (
     normpath,
     expanduser,
 )
-from .timestamp import Timestamp
+from .timestamp import Timestamp, SILENT_D, silentConvert
 from .prepare import (
     levels,
     order,
@@ -40,6 +42,7 @@ from .prepare import (
     levUp,
     levDown,
     boundary,
+    characters,
     sections,
     structure,
 )
@@ -62,20 +65,21 @@ from ..convert.mql import MQL, tfFromMql
 OTEXT_DEFAULT = dict(sectionFeatures="", sectionTypes="")
 
 PRECOMPUTE = (
-    (False, "__levels__", levels, (OTYPE, OSLOTS, OTEXT)),
-    (False, "__order__", order, (OTYPE, OSLOTS) + ("__levels__",)),
-    (False, "__rank__", rank, (OTYPE, "__order__")),
-    (False, "__levUp__", levUp, (OTYPE, OSLOTS) + ("__rank__",)),
-    (False, "__levDown__", levDown, (OTYPE, "__levUp__", "__rank__")),
-    (False, "__boundary__", boundary, (OTYPE, OSLOTS) + ("__rank__",)),
+    (0, "__levels__", levels, (OTYPE, OSLOTS, OTEXT)),
+    (0, "__order__", order, (OTYPE, OSLOTS) + ("__levels__",)),
+    (0, "__rank__", rank, (OTYPE, "__order__")),
+    (0, "__levUp__", levUp, (OTYPE, OSLOTS) + ("__rank__",)),
+    (0, "__levDown__", levDown, (OTYPE, "__levUp__", "__rank__")),
+    (1, "__characters__", characters, (OTEXT,)),
+    (0, "__boundary__", boundary, (OTYPE, OSLOTS) + ("__rank__",)),
     (
-        True,
+        2,
         "__sections__",
         sections,
         (OTYPE, OSLOTS, OTEXT) + ("__levUp__", "__levels__"),
     ),
     (
-        True,
+        2,
         "__structure__",
         structure,
         (OTYPE, OSLOTS, OTEXT)
@@ -165,10 +169,13 @@ class FabricCore(object):
         So if you leave it out, Text-Fabric will just search the paths specified
         in `locations`.
 
-    silent:
-        If `True` is passed, banners and normal progress messages are suppressed.
-        If `'deep'` is passed, all informational and warning messages are suppressed.
-        Errors still pass through.
+    silent: string, optional `tf.core.timestamp.SILENT_D`
+        See `tf.core.timestamp.Timestamp`
+
+    _withGc: boolean, optional True
+        If False, it disables the Python garbage collector before
+        loading features. Used to experiment with performance.
+
 
     !!! note "otext@ in modules"
         If modules contain features with a name starting with `otext@`, then the format
@@ -183,10 +190,11 @@ class FabricCore(object):
         An object from which you can call up all the of methods of the core API.
     """
 
-    def __init__(self, locations=None, modules=None, silent=False):
-
+    def __init__(self, locations=None, modules=None, silent=SILENT_D, _withGc=True):
+        silent = silentConvert(silent)
+        self._withGc = _withGc
         self.silent = silent
-        tmObj = Timestamp()
+        tmObj = Timestamp(silent=silent)
         self.tmObj = tmObj
         setSilent = tmObj.setSilent
         setSilent(silent)
@@ -264,7 +272,7 @@ Api reference : {APIREF}
 
         self._makeIndex()
 
-    def load(self, features, add=False, silent=None):
+    def load(self, features, add=False, silent=SILENT_D):
         """Loads features from disk into RAM memory.
 
         Parameters
@@ -280,13 +288,8 @@ Api reference : {APIREF}
             by the current API.
             Meant to be able to dynamically load features without reloading lots
             of features for nothing.
-        silent: boolean, optional `None`
-            If `False`, the features will be loaded rather silently,
-            most messages will be suppressed.
-            Time consuming operations will always be announced,
-            so that you know what Text-Fabric is doing.
-            If `True` is passed, all informational messages will be suppressed.
-            This is handy I you want to load data as part of other methods, on-the-fly.
+        silent: string, optional `tf.core.timestamp.SILENT_D`
+            See `tf.core.timestamp.Timestamp`
 
         Returns
         -------
@@ -296,6 +299,7 @@ Api reference : {APIREF}
             if the feature could be loaded, else `False`.
         """
 
+        silent = silentConvert(silent)
         tmObj = self.tmObj
         isSilent = tmObj.isSilent
         setSilent = tmObj.setSilent
@@ -305,14 +309,15 @@ Api reference : {APIREF}
         error = tmObj.error
         cache = tmObj.cache
         reset = tmObj.reset
+        featuresOnly = self.featuresOnly
 
-        if silent is not None:
-            wasSilent = isSilent()
-            setSilent(silent)
+        wasSilent = isSilent()
+        setSilent(silent)
         indent(level=0, reset=True)
         self.sectionsOK = True
         self.structureOK = True
         self.good = True
+
         if self.good:
             featuresRequested = sorted(fitemize(features))
             if add:
@@ -320,9 +325,11 @@ Api reference : {APIREF}
             else:
                 self.featuresRequested = featuresRequested
             for fName in (OTYPE, OSLOTS, OTEXT):
-                self._loadFeature(fName, optional=fName == OTEXT)
-        if self.good:
-            self.textFeatures = set()
+                self._loadFeature(fName, optional=fName == OTEXT or featuresOnly)
+
+        self.textFeatures = set()
+
+        if self.good and not featuresOnly:
             if OTEXT in self.features:
                 otextMeta = self.features[OTEXT].metaData
                 for otextMod in self.features:
@@ -336,6 +343,7 @@ Api reference : {APIREF}
                 )
                 self.structureTypes = itemize(otextMeta.get("structureTypes", ""), ",")
                 (self.cformats, self.formatFeats) = collectFormats(otextMeta)
+
                 if not (0 < len(self.sectionTypes) <= 3) or not (
                     0 < len(self.sectionFeats) <= 3
                 ):
@@ -370,12 +378,30 @@ Api reference : {APIREF}
                 for fName in self.textFeatures:
                     self._loadFeature(fName, optional=fName in formatFeats)
 
+                dep1Feats = self.dep1Feats
+                if dep1Feats:
+                    cformats = self.cformats
+                    tFormats = {}
+                    tFeats = set()
+                    for (fmt, (otpl, tpl, featData)) in cformats.items():
+                        feats = set(chain.from_iterable(x[0] for x in featData))
+                        tFormats[fmt] = tuple(sorted(feats))
+                        tFeats |= feats
+                    tFeats = tuple(sorted(tFeats))
+                    extraDependencies = [tFormats]
+                    for tFeat in tFeats:
+                        featData = self.features[tFeat].data
+                        extraDependencies.append((tFeat, featData))
+                    for cFeat in dep1Feats:
+                        self.features[cFeat].dependencies += extraDependencies
+
             else:
                 self.sectionsOK = False
                 self.structureOK = False
 
-        if self.good:
+        if self.good and not featuresOnly:
             self._precompute()
+
         if self.good:
             reset()
             for fName in self.featuresRequested:
@@ -387,6 +413,7 @@ Api reference : {APIREF}
                     result = False
                     break
                 reset()
+
         indent(level=0)
         if self.good:
             if add:
@@ -404,18 +431,17 @@ Api reference : {APIREF}
                     result = False
         else:
             result = False
-        if silent is not None:
-            setSilent(wasSilent)
+
+        setSilent(wasSilent)
         return result
 
-    def explore(self, silent=None, show=True):
+    def explore(self, silent=SILENT_D, show=True):
         """Makes categorization of all features in the dataset.
 
         Parameters
         ----------
-        silent: boolean, optional `None`
-            If `False` a message containing the total numbers of features
-            is issued.
+        silent: string, optional `tf.core.timestamp.SILENT_D`
+            See `tf.core.timestamp.Timestamp`
         show: boolean, optional `True`
             If `False`, the resulting dictionary is delivered in `TF.featureSets`;
             if `True`, the dictionary is returned as function result.
@@ -441,20 +467,20 @@ Api reference : {APIREF}
         `tf.core.api.Api.Fall` for nodes and `tf.core.api.Api.Eall` for edges.
         """
 
+        silent = silentConvert(silent)
         tmObj = self.tmObj
         isSilent = tmObj.isSilent
         setSilent = tmObj.setSilent
         info = tmObj.info
 
-        if silent is not None:
-            wasSilent = isSilent()
-            setSilent(silent)
+        wasSilent = isSilent()
+        setSilent(silent)
         nodes = set()
         edges = set()
         configs = set()
         computeds = set()
         for (fName, fObj) in self.features.items():
-            fObj.load(metaOnly=True)
+            fObj.load(silent=silent, metaOnly=True)
             dest = None
             if fObj.method:
                 dest = computeds
@@ -476,8 +502,7 @@ Api reference : {APIREF}
         self.featureSets = dict(
             nodes=nodes, edges=edges, configs=configs, computeds=computeds
         )
-        if silent is not None:
-            setSilent(wasSilent)
+        setSilent(wasSilent)
         if show:
             return dict(
                 (kind, tuple(sorted(kindSet)))
@@ -486,17 +511,16 @@ Api reference : {APIREF}
                 )
             )
 
-    def loadAll(self, silent=None):
+    def loadAll(self, silent=SILENT_D):
         """Load all loadable features.
 
         Parameters
         ----------
-        silent: boolean, optional `None`
-            TF is silent if you specified `silent=True` in a preceding
-            `TF=Fabric()` call.
-            But if you did not, you can also pass `silent=True` to this call.
+        silent: string, optional `tf.core.timestamp.SILENT_D`
+            See `tf.core.timestamp.Timestamp`
         """
 
+        silent = silentConvert(silent)
         api = self.load("", silent=silent)
         allFeatures = self.explore(silent=silent, show=True)
         loadableFeatures = allFeatures["nodes"] + allFeatures["edges"]
@@ -535,7 +559,7 @@ Api reference : {APIREF}
         metaData={},
         location=None,
         module=None,
-        silent=None,
+        silent=SILENT_D,
     ):
         """Saves newly generated data to disk as TF features, nodes and/or edges.
 
@@ -561,7 +585,7 @@ Api reference : {APIREF}
             as values.
             This parameter holds all those dictionaries, keyed by feature name.
 
-        metadata: dict of  dict
+        metaData: dict of  dict
             The meta data for every feature to be saved is a key-value dictionary.
             This parameter holds all those dictionaries, keyed by feature name.
 
@@ -611,12 +635,11 @@ Api reference : {APIREF}
             If you pass `location=path1` and `module=path2`,
             TF will save in `path1/path2`.
 
-        silent: boolean, optional `None`
-            TF is silent if you specified `silent=True` in a preceding
-            `TF=Fabric()` call.
-            But if you did not, you can also pass `silent=True` to this call.
+        silent: string, optional `tf.core.timestamp.SILENT_D`
+            See `tf.core.timestamp.Timestamp`
         """
 
+        silent = silentConvert(silent)
         tmObj = self.tmObj
         isSilent = tmObj.isSilent
         setSilent = tmObj.setSilent
@@ -625,9 +648,8 @@ Api reference : {APIREF}
         error = tmObj.error
 
         good = True
-        if silent is not None:
-            wasSilent = isSilent()
-            setSilent(silent)
+        wasSilent = isSilent()
+        setSilent(silent)
         indent(level=0, reset=True)
         self._getWriteLoc(location=location, module=module)
         configFeatures = dict(
@@ -738,7 +760,7 @@ Api reference : {APIREF}
                 edgeValues=edgeValues,
             )
             tag = "config" if isConfig else "edge" if isEdge else "node"
-            if fObj.save(nodeRanges=fName == OTYPE, overwrite=True):
+            if fObj.save(nodeRanges=fName == OTYPE, overwrite=True, silent=silent):
                 total[tag] += 1
             else:
                 failed[tag] += 1
@@ -754,8 +776,7 @@ Api reference : {APIREF}
                 error(f"Failed to export {nf} {tag} features")
             good = False
 
-        if silent is not None:
-            setSilent(wasSilent)
+        setSilent(wasSilent)
         return good
 
     def exportMQL(self, mqlName, mqlDir):
@@ -875,8 +896,7 @@ Api reference : {APIREF}
                 error(f'Feature "{fName}" not available in\n{self.locationRep}')
                 self.good = False
         else:
-            # if not self.features[fName].load(silent=silent or (fName not in self.featuresRequested)):
-            if not self.features[fName].load(silent=silent):
+            if not self.features[fName].load(silent=silent, _withGc=self._withGc):
                 self.good = False
 
     def _makeIndex(self):
@@ -914,55 +934,59 @@ Api reference : {APIREF}
             tm=False,
         )
 
-        good = True
-        for fName in (OTYPE, OSLOTS, OTEXT):
-            if fName not in self.features:
-                if fName == OTEXT:
-                    info(
-                        (
-                            f'Warp feature "{OTEXT}" not found. Working without Text-API\n'
-                        )
-                    )
-                    self.features[OTEXT] = Data(
-                        f"{OTEXT}.tf",
-                        self.tmObj,
-                        isConfig=True,
-                        metaData=OTEXT_DEFAULT,
-                    )
-                    self.features[OTEXT].dataLoaded = True
-                else:
-                    info(f'Warp feature "{fName}" not found in\n{self.locationRep}')
-                    good = False
-            elif fName == OTEXT:
-                self._loadFeature(fName, optional=True)
-        if not good:
-            return False
-        self.warpDir = self.features[OTYPE].dirName
-        self.precomputeList = []
-        for (dep2, fName, method, dependencies) in PRECOMPUTE:
-            thisGood = True
-            if dep2 and OTEXT not in self.features:
-                continue
-            if dep2:
-                otextMeta = self.features[OTEXT].metaData
-                sFeatures = f"{KIND[fName]}Features"
-                sFeats = tuple(itemize(otextMeta.get(sFeatures, ""), ","))
-                dependencies = dependencies + sFeats
-            for dep in dependencies:
-                if dep not in self.features:
-                    warning(
-                        f'Missing dependency for computed data feature "{fName}": "{dep}"'
-                    )
-                    thisGood = False
-            if not thisGood:
-                good = False
-            self.features[fName] = Data(
-                f"{self.warpDir}/{fName}.x",
-                self.tmObj,
-                method=method,
-                dependencies=[self.features.get(dep, None) for dep in dependencies],
+        self.featuresOnly = False
+
+        if OTYPE not in self.features or OSLOTS not in self.features:
+            info(
+                f"Not all of the warp features {OTYPE} and {OSLOTS} "
+                f"are present in\n{self.locationRep}"
             )
-            self.precomputeList.append((fName, dep2))
+            info("Only the Feature and Edge APIs will be enabled")
+            self.featuresOnly = True
+        if OTEXT in self.features:
+            self._loadFeature(OTEXT, optional=True)
+        else:
+            info((f'Warp feature "{OTEXT}" not found. Working without Text-API\n'))
+            self.features[OTEXT] = Data(
+                f"{OTEXT}.tf",
+                self.tmObj,
+                isConfig=True,
+                metaData=OTEXT_DEFAULT,
+            )
+            self.features[OTEXT].dataLoaded = True
+
+        good = True
+        if not self.featuresOnly:
+            self.warpDir = self.features[OTYPE].dirName
+            self.precomputeList = []
+            self.dep1Feats = []
+            for (dep2, fName, method, dependencies) in PRECOMPUTE:
+                thisGood = True
+                if dep2 and OTEXT not in self.features:
+                    continue
+                if dep2 == 1:
+                    self.dep1Feats.append(fName)
+                elif dep2 == 2:
+                    otextMeta = self.features[OTEXT].metaData
+                    sFeatures = f"{KIND[fName]}Features"
+                    sFeats = tuple(itemize(otextMeta.get(sFeatures, ""), ","))
+                    dependencies = dependencies + sFeats
+                for dep in dependencies:
+                    if dep not in self.features:
+                        warning(
+                            "Missing dependency for computed data feature "
+                            f'"{fName}": "{dep}"'
+                        )
+                        thisGood = False
+                if not thisGood:
+                    good = False
+                self.features[fName] = Data(
+                    f"{self.warpDir}/{fName}.x",
+                    self.tmObj,
+                    method=method,
+                    dependencies=[self.features.get(dep, None) for dep in dependencies],
+                )
+                self.precomputeList.append((fName, dep2))
         self.good = good
 
     def _getWriteLoc(self, location=None, module=None):
@@ -988,14 +1012,14 @@ Api reference : {APIREF}
 
     def _precompute(self):
         tmObj = self.tmObj
-        info = tmObj.info
+        isSilent = tmObj.isSilent
         good = True
+
         for (fName, dep2) in self.precomputeList:
             ok = getattr(self, f'{fName.strip("_")}OK', False)
-            if dep2 and not ok:
+            if dep2 == 2 and not ok:
                 continue
-            info(f"... {fName} ...")
-            if not self.features[fName].load():
+            if not self.features[fName].load(silent=isSilent()):
                 good = False
                 break
         self.good = good
@@ -1008,15 +1032,19 @@ Api reference : {APIREF}
         isSilent = tmObj.isSilent
         indent = tmObj.indent
         info = tmObj.info
+        featuresOnly = self.featuresOnly
 
         silent = isSilent()
         api = Api(self)
+        api.featuresOnly = featuresOnly
 
-        w0info = self.features[OTYPE]
-        w1info = self.features[OSLOTS]
+        if not featuresOnly:
+            w0info = self.features[OTYPE]
+            w1info = self.features[OSLOTS]
 
-        setattr(api.F, OTYPE, OtypeFeature(api, w0info.metaData, w0info.data))
-        setattr(api.E, OSLOTS, OslotsFeature(api, w1info.metaData, w1info.data))
+        if not featuresOnly:
+            setattr(api.F, OTYPE, OtypeFeature(api, w0info.metaData, w0info.data))
+            setattr(api.E, OSLOTS, OslotsFeature(api, w1info.metaData, w1info.data))
 
         requestedSet = set(self.featuresRequested)
 
@@ -1024,15 +1052,20 @@ Api reference : {APIREF}
             fObj = self.features[fName]
             if fObj.dataLoaded and not fObj.isConfig:
                 if fObj.method:
-                    feat = fName.strip("_")
-                    ok = getattr(self, f"{feat}OK", False)
-                    ap = api.C
-                    if fName in [x[0] for x in self.precomputeList if not x[1] or ok]:
-                        setattr(ap, feat, Computed(api, fObj.data))
-                    else:
-                        fObj.unload()
-                        if hasattr(ap, feat):
-                            delattr(api.C, feat)
+                    if not featuresOnly:
+                        feat = fName.strip("_")
+                        ok = getattr(self, f"{feat}OK", False)
+                        ap = api.C
+                        if fName in [
+                            fn
+                            for (fn, dep2) in self.precomputeList
+                            if not dep2 == 2 or ok
+                        ]:
+                            setattr(ap, feat, Computed(api, fObj.data))
+                        else:
+                            fObj.unload()
+                            if hasattr(ap, feat):
+                                delattr(api.C, feat)
                 else:
                     if fName in requestedSet | self.textFeatures:
                         if fName in (OTYPE, OSLOTS, OTEXT):
@@ -1062,11 +1095,12 @@ Api reference : {APIREF}
                             if hasattr(api.F, fName):
                                 delattr(api.F, fName)
                         fObj.unload()
-        addOtype(api)
-        addNodes(api)
-        addLocality(api)
-        addText(api)
-        addSearch(api, silent)
+        if not featuresOnly:
+            addOtype(api)
+            addNodes(api)
+            addLocality(api)
+            addText(api)
+            addSearch(api, silent)
         indent(level=0)
         info("All features loaded/computed - for details use TF.isLoaded()")
         self.api = api
