@@ -11,9 +11,9 @@ the order and nature of the steps is configured in
 The functions in this module implement those tasks.
 """
 
-from array import array
 import collections
 import functools
+from array import array
 from .helpers import itemize
 
 
@@ -134,14 +134,9 @@ def order(info, error, otype, oslots, levels):
 
     Returns
     -------
-    array
+    tuple
         All nodes, slot and nonslot, in canonical order.
 
-    Notes
-    -----
-    We store the result in an array because it saves a lot of memory, and access
-    is still fast.
-
     See Also
     --------
     tf.core.nodes: canonical ordering
@@ -184,12 +179,13 @@ def order(info, error, otype, oslots, levels):
             return 1
         am = min(sa - sb)
         bm = min(sb - sa)
-        return -1 if am < bm else 1 if bm < am else None
+        return -1 if am < bm else 1 if bm < am else 0
 
     canonKey = functools.cmp_to_key(before)
     info("sorting nodes")
     nodes = sorted(range(1, maxNode + 1), key=canonKey)
-    return array("I", nodes)
+    # return array("I", nodes)
+    return tuple(nodes)
 
 
 def rank(info, error, otype, order):
@@ -206,24 +202,20 @@ def rank(info, error, otype, order):
         Method to write error messages to the console.
     otype: iterable
         The data of the *otype* feature.
-    order: array
+    order: tuple
         The data of the *order* feature.
 
     Returns
     -------
-    array
+    tuple
         The ranks of all nodes, slot and nonslot, with respect to the canonical order.
-
-    Notes
-    -----
-    We store the result in an array because it saves a lot of memory, and access
-    is still fast.
     """
 
     (otype, maxSlot, maxNode, slotType) = otype
     info("ranking nodes")
     nodesRank = dict(((n, i) for (i, n) in enumerate(order)))
     return array("I", (nodesRank[n] for n in range(1, maxNode + 1)))
+    # return tuple((nodesRank[n] for n in range(1, maxNode + 1)))
 
 
 def levUp(info, error, otype, oslots, rank):
@@ -244,20 +236,20 @@ def levUp(info, error, otype, oslots, rank):
         The data of the *otype* feature.
     oslots: iterable
         The data of the *oslots* feature.
-    rank: array
+    rank: tuple
         The data of the *rank* precompute step.
 
     Returns
     -------
     tuple
-        The n-th member is an array of the embedder nodes of n.
-        Those arrays are sorted in canonical order (`tf.core.nodes`).
+        The n-th member is an tuple of the embedder nodes of n.
+        Those tuples are sorted in canonical order (`tf.core.nodes`).
 
     Notes
     -----
     !!! hint "Memory efficiency"
-        Many nodes have the same array of embedders.
-        Those embedder arrays will be reused for those nodes.
+        Many nodes have the same tuple of embedders.
+        Those embedder tuples will be reused for those nodes.
 
     Warnings
     --------
@@ -307,7 +299,8 @@ def levUp(info, error, otype, oslots, rank):
     embeddersx = []
     for t in embedders:
         if t not in seen:
-            seen[t] = array("I", t)
+            # seen[t] = array("I", t)
+            seen[t] = tuple(t)
         embeddersx.append(seen[t])
     return tuple(embeddersx)
 
@@ -330,14 +323,14 @@ def levDown(info, error, otype, levUp, rank):
         The data of the *otype* feature.
     levUp: iterable
         The data of the *levUp* precompute step.
-    rank: array
+    rank: tuple
         The data of the *rank* precompute step.
 
     Returns
     -------
     tuple
-        The *n*-th member is an array of the embedded nodes of *n + maxSlot*.
-        Those arrays are sorted in canonical order (`tf.core.nodes`).
+        The *n*-th member is an tuple of the embedded nodes of *n + maxSlot*.
+        Those tuples are sorted in canonical order (`tf.core.nodes`).
 
     !!! hint "Memory efficiency"
         Slot nodes do not have embedded nodes, so they do not have to occupy
@@ -363,10 +356,66 @@ def levDown(info, error, otype, levUp, rank):
     for n in range(maxSlot + 1, maxNode + 1):
         embeddees.append(
             array("I", sorted(inverse.get(n, []), key=lambda m: rank[m - 1]))
+            # tuple(sorted(inverse.get(n, []), key=lambda m: rank[m - 1]))
         )
     return tuple(embeddees)
 
 
+def characters(info, error, otext, tFormats, *tFeats):
+    """Computes character data.
+
+    For each text format, a frequency list of the characters in that format
+    is made.
+
+    Parameters
+    ----------
+    info: function
+        Method to write informational messages to the console.
+    error: function
+        Method to write error messages to the console.
+    otext: iterable
+        The data of the *otext* feature.
+    tFormats:
+        Dictionary keyed by text format and valued by the tuple of features
+        used in that format.
+    tFeats: iterable
+        Each tFeat is the name and the data of a text feature.
+        i.e. a feature used in text formats.
+
+    Returns
+    -------
+    dict
+        Keyed by format valued by a frequency dict, which is
+        itself keyed by single characters and valued by the frequency
+        of that character in the whole corpus when rendered with that format.
+    """
+
+    charFreqsByFeature = {}
+
+    for (tFeat, data) in tFeats:
+        freqList = collections.Counter()
+        if data is not None:
+            for v in data.values():
+                freqList[v] += 1
+        charFreq = collections.defaultdict(lambda: 0)
+        for (v, freq) in freqList.items():
+            for c in str(v):
+                charFreq[c] += freq
+        charFreqsByFeature[tFeat] = charFreq
+
+    charFreqsByFmt = {}
+
+    for (fmt, tFeatures) in sorted(tFormats.items()):
+        charFreq = collections.defaultdict(lambda: 0)
+        for tFeat in tFeatures:
+            thisCharFreq = charFreqsByFeature[tFeat]
+            for (c, freq) in thisCharFreq.items():
+                charFreq[c] += freq
+        charFreqsByFmt[fmt] = sorted(x for x in charFreq.items())
+
+    return charFreqsByFmt
+
+
 def boundary(info, error, otype, oslots, rank):
     """Computes boundary data.
 
@@ -388,17 +437,17 @@ def boundary(info, error, otype, oslots, rank):
         The data of the *otype* feature.
     oslots: iterable
         The data of the *oslots* feature.
-    rank: array
+    rank: tuple
         The data of the *rank* precompute step.
 
     Returns
     -------
     tuple
-        *   first: tuple of array
-            The *n*-th member is the array of nodes that start at slot *n*,
+        *   first: tuple of tuple
+            The *n*-th member is the tuple of nodes that start at slot *n*,
             ordered in *reversed* canonical order (`tf.core.nodes`);
-        *   last: tuple of array
-            The *n*-th member is the array of nodes that end at slot *n*,
+        *   last: tuple of tuple
+            The *n*-th member is the tuple of nodes that end at slot *n*,
             ordered in canonical order;
 
     Notes
@@ -411,19 +460,23 @@ def boundary(info, error, otype, oslots, rank):
     oslots = oslots[0]
     firstSlotsD = {}
     lastSlotsD = {}
-    for (k, mList) in enumerate(oslots):
-        firstSlotsD.setdefault(mList[0], []).append(k + 1 + maxSlot)
-        lastSlotsD.setdefault(mList[-1], []).append(k + 1 + maxSlot)
-    firstSlots = []
-    lastSlots = []
-    for n in range(1, maxSlot + 1):
-        firstSlots.append(
-            array("I", sorted(firstSlotsD.get(n, []), key=lambda k: -rank[k - 1]))
-        )
-        lastSlots.append(
-            array("I", sorted(lastSlotsD.get(n, []), key=lambda k: rank[k - 1]))
-        )
-    return (tuple(firstSlots), tuple(lastSlots))
+
+    for (node, slots) in enumerate(oslots):
+        realNode = node + 1 + maxSlot
+        firstSlotsD.setdefault(slots[0], []).append(realNode)
+        lastSlotsD.setdefault(slots[-1], []).append(realNode)
+
+    firstSlots = tuple(
+        tuple(sorted(firstSlotsD.get(n, []), key=lambda node: -rank[node - 1]))
+        # array("I", sorted(firstSlotsD.get(n, []), key=lambda node: -rank[node - 1]))
+        for n in range(1, maxSlot + 1)
+    )
+    lastSlots = tuple(
+        tuple(sorted(lastSlotsD.get(n, []), key=lambda node: rank[node - 1]))
+        # array("I", sorted(lastSlotsD.get(n, []), key=lambda node: rank[node - 1]))
+        for n in range(1, maxSlot + 1)
+    )
+    return (firstSlots, lastSlots)
 
 
 def sections(info, error, otype, oslots, otext, levUp, levels, *sFeats):
@@ -453,12 +506,12 @@ def sections(info, error, otype, oslots, otext, levUp, levels, *sFeats):
         The data of the *oslots* feature.
     otext: iterable
         The data of the *otext* feature.
-    levUp: array
+    levUp: tuple
         The data of the *levUp* precompute step.
-    levels: array
+    levels: tuple
         The data of the *levels* precompute step.
     sFeats: iterable
-        The names of section features.
+        Each sFeat is the data of a section feature.
 
     Returns
     -------
@@ -481,7 +534,7 @@ def sections(info, error, otype, oslots, otext, levUp, levels, *sFeats):
 
     (otype, maxSlot, maxNode, slotType) = otype
     oslots = oslots[0]
-    support = dict(((o[0], (o[2], o[3])) for o in levels))
+    support = {level[0]: (level[2], level[3]) for level in levels}
     sTypes = itemize(otext["sectionTypes"], ",")
     sec1 = {}
     sec2 = {}
@@ -501,11 +554,15 @@ def sections(info, error, otype, oslots, otext, levUp, levels, *sFeats):
                 ] += 1
                 continue
             n0 = n0s[0]
-            n1s = sFeats[1][n1]
+            n1head = sFeats[1].get(n1, None)
+            if n1head is None:
+                nestingProblems[
+                    f"{sTypes[1]}-node {n1} has no section heading"
+                ] += 1
             if n0 not in sec1:
                 sec1[n0] = {}
-            if n1s not in sec1[n0]:
-                sec1[n0][n1s] = n1
+            if n1head not in sec1[n0]:
+                sec1[n0][n1head] = n1
                 c1 += 1
         info(f"{c1} {sTypes[1]}s indexed")
 
@@ -521,22 +578,32 @@ def sections(info, error, otype, oslots, otext, levUp, levels, *sFeats):
                 ] += 1
                 continue
             n0 = n0s[0]
+
             n1s = tuple(x for x in levUp[n2 - 1] if otype[x - maxSlot - 1] == sTypes[1])
             if not n1s:
                 nestingProblems[
                     f"section {sTypes[2]} without containing {sTypes[1]}"
                 ] += 1
-                print(f"{sTypes[2]} {n2} without containing {sTypes[1]}")
                 continue
             n1 = n1s[0]
-            n1s = sFeats[1][n1]
-            n2s = sFeats[2][n2]
+
+            n1head = sFeats[1].get(n1, None)
+            if n1head is None:
+                nestingProblems[
+                    f"{sTypes[1]}-node {n1} has no section heading"
+                ] += 1
+            n2head = sFeats[2].get(n2, None)
+            if n2head is None:
+                nestingProblems[
+                    f"{sTypes[2]}-node {n2} has no section heading"
+                ] += 1
+
             if n0 not in sec1:
                 sec1[n0] = {}
-            if n1s not in sec1[n0]:
-                sec1[n0][n1s] = n1
+            if n1head not in sec1[n0]:
+                sec1[n0][n1head] = n1
                 c1 += 1
-            sec2.setdefault(n0, {}).setdefault(n1s, {})[n2s] = n2
+            sec2.setdefault(n0, {}).setdefault(n1head, {})[n2head] = n2
             c2 += 1
         info(f"{c1} {sTypes[1]}s and {c2} {sTypes[2]}s indexed")
 
@@ -575,12 +642,12 @@ def structure(info, error, otype, oslots, otext, rank, levUp, *sFeats):
         The data of the *oslots* feature.
     otext: iterable
         The data of the *otext* feature.
-    rank: array
+    rank: tuple
         The data of the *rank* precompute step.
-    levUp: array
+    levUp: tuple
         The data of the *levUp* precompute step.
     sFeats: iterable
-        The names of structural features.
+        Each sFeat the data of a section feature.
 
     Returns
     -------
