@@ -5,25 +5,27 @@
 from ..core.helpers import console, wrapMessages
 from .searchexe import SearchExe
 from ..parameters import YARN_RATIO, TRY_LIMIT_FROM, TRY_LIMIT_TO
+from ..core.timestamp import SILENT_D, AUTO, silentConvert
 
 
 class Search(object):
-    """
+    """ """
 
-    """
-
-    def __init__(self, api, silent):
+    def __init__(self, api, silent=SILENT_D):
+        silent = silentConvert(silent)
         self.api = api
         self.silent = silent
         self.exe = None
         self.perfDefaults = dict(
-            yarnRatio=YARN_RATIO, tryLimitFrom=TRY_LIMIT_FROM, tryLimitTo=TRY_LIMIT_TO,
+            yarnRatio=YARN_RATIO,
+            tryLimitFrom=TRY_LIMIT_FROM,
+            tryLimitTo=TRY_LIMIT_TO,
         )
         self.perfParams = {}
         self.perfParams.update(self.perfDefaults)
         SearchExe.setPerfParams(self.perfParams)
 
-    def tweakPerformance(self, silent=False, **kwargs):
+    def tweakPerformance(self, silent=SILENT_D, **kwargs):
         """Tweak parameters that influence the search process.
 
         !!! explanation "Theory"
@@ -132,6 +134,7 @@ class Search(object):
             See `tryLimitFrom`
         """
 
+        silent = silentConvert(silent)
         api = self.api
         TF = api.TF
         error = TF.error
@@ -167,7 +170,7 @@ class Search(object):
         limit=None,
         sets=None,
         shallow=False,
-        silent=True,
+        silent=SILENT_D,
         here=True,
         _msgCache=False,
     ):
@@ -193,7 +196,15 @@ class Search(object):
             If not `None`, it should be a dictionary of sets, keyed by a names.
 
         limit: integer, optional `None`
-            If `limit` is a number, it will fetch only that many results.
+            If `limit` is a positive number, it will fetch only that many results.
+            If it is negative, 0, None, or absent, it will fetch arbitrary many results.
+
+            !!! caution "there is an upper *fail limit* for safety reasons.
+                The limit is a factor times the max node in your corpus.
+                See `tf.parameters.SEARCH_FAIL_FACTOR`.
+                If this *fail limit* is exceeded in cases where no positive `limit`
+                has been passed, you get a warning message.
+
         Returns
         -------
         generator | tuple
@@ -230,11 +241,18 @@ class Search(object):
         queryResults = exe.search(limit=limit)
         if type(_msgCache) is list:
             messages = wrapMessages(_msgCache)
+            self._msgCache = _msgCache
             return (queryResults, messages) if here else (queryResults, messages, exe)
         return queryResults
 
     def study(
-        self, searchTemplate, strategy=None, sets=None, shallow=False, here=True,
+        self,
+        searchTemplate,
+        strategy=None,
+        sets=None,
+        shallow=False,
+        here=True,
+        silent=SILENT_D,
     ):
         """Studies a template to prepare for searching with it.
 
@@ -276,14 +294,17 @@ class Search(object):
             If not `None`, it should be a dictionary of sets, keyed by a names.
             In the search template you can refer to those names to invoke those sets.
 
-        silent: boolean, optional `None`
-            If you want to suppress most of the output, say `silent=True`.
+        silent: string, optional `tf.core.timestamp.SILENT_D`
+            See `tf.core.timestamp.Timestamp`
 
         See Also
         --------
         tf.about.searchusage: Search guide
         """
 
+        if silent is False:
+            silent = AUTO
+
         exe = SearchExe(
             self.api,
             searchTemplate,
@@ -292,7 +313,7 @@ class Search(object):
             offset=0,
             sets=sets,
             shallow=shallow,
-            silent=False,
+            silent=SILENT_D,
             showQuantifiers=True,
             setInfo={},
         )
@@ -310,7 +331,16 @@ class Search(object):
         ----------
 
         limit: integer, optional `None`
-            If `limit` is a number, it will fetch only that many results.
+            If `limit` is a positive number, it will fetch only that many results.
+            If it is negative, 0, None, or absent, it will fetch arbitrary many results.
+
+            !!! caution "there is an upper *fail limit* for safety reasons.
+                The limit is a factor times the max node in your corpus.
+                See `tf.parameters.SEARCH_FAIL_FACTOR`.
+                If this *fail limit* is exceeded in cases where no positive `limit`
+                has been passed, you get a warning message.
+
+
         Returns
         -------
         generator | tuple
@@ -370,10 +400,15 @@ class Search(object):
             Every once for every `progress` results a progress message is shown
             when fetching results.
 
-        limit: integer, optional, default `1000`
-            Fetch results up to this `limit`.
-            Setting `limit` to 0 or a negative value means no limit: all results will be
-            counted.
+        limit: integer, optional `None`
+            If `limit` is a positive number, it will fetch only that many results.
+            If it is negative, 0, None, or absent, it will fetch arbitrary many results.
+
+            !!! caution "there is an upper *fail limit* for safety reasons.
+                The limit is a factor times the max node in your corpus.
+                See `tf.parameters.SEARCH_FAIL_FACTOR`.
+                If this *fail limit* is exceeded in cases where no positive `limit`
+                has been passed, you get a warning message.
 
         !!! note "why needed"
             You typically need this in cases where result fetching turns out to
@@ -501,7 +536,9 @@ class Search(object):
                 field = ""
             else:
                 field = "{}[{}{}]".format(
-                    otype, T.text(words[0:5]), "..." if len(words) > 5 else "",
+                    otype,
+                    T.text(words[0:5]),
+                    "..." if len(words) > 5 else "",
                 )
             fields.append(field)
         return " ".join(fields)
