@@ -34,7 +34,7 @@ The TF kernel can be connected by an other Python program as follows:
 
 ```python
 from tf.server.kernel import makeTfConnection
-TF = makeTfConnection(host, port)
+TF = makeTfConnection(lhost, port)
 api = TF.connect()
 ```
 
@@ -82,12 +82,14 @@ from functools import reduce
 import rpyc
 from rpyc.utils.server import ThreadedServer
 
+from ..parameters import GH
 from ..core.helpers import console
 from ..advanced.app import findApp
 from ..advanced.highlight import getPassageHighlights
 from ..advanced.search import runSearch, runSearchCondensed
 from ..advanced.helpers import getRowsX
 from ..advanced.tables import compose, composeP, composeT
+from ..advanced.text import specialCharacters
 
 from .command import argKernel
 
@@ -155,8 +157,19 @@ def makeTfKernel(app, appName, port):
 
             app = self.app
             aContext = app.context
+            backend = app.backend
+            org = aContext.org
+            repo = aContext.repo
             commit = aContext.commit
-            appProvenance = ((("name", appName), ("commit", commit)),)
+            appProvenance = (
+                (
+                    ("backend", backend),
+                    ("name", appName),
+                    ("org", org),
+                    ("repo", repo),
+                    ("commit", commit),
+                ),
+            )
             return (appProvenance, app.provenance)
 
         def exposed_setNames(self):
@@ -182,6 +195,12 @@ def makeTfKernel(app, appName, port):
             app = self.app
             return f'<style type="text/css">{app.loadCss()}</style>'
 
+        def exposed_characters(self, fmt=None):
+            """Delivers the HTML for a widget of hard-to-type characters."""
+
+            app = self.app
+            return specialCharacters(app, fmt=fmt, _browse=True)
+
         def exposed_context(self):
             """Fetches the TF app context settings for the corpus."""
 
@@ -447,7 +466,7 @@ def makeTfKernel(app, appName, port):
             total = 0
 
             results = ()
-            messages = ""
+            messages = ("", "")
             if query:
                 (results, messages, features) = (
                     runSearchCondensed(app, query, cache, condenseType)
@@ -455,7 +474,7 @@ def makeTfKernel(app, appName, port):
                     else runSearch(app, query, cache)
                 )
 
-                if messages:
+                if messages[0]:
                     results = ()
                 total += len(results)
 
@@ -487,7 +506,7 @@ def makeTfKernel(app, appName, port):
                 getx=getx,
                 **options,
             )
-            return (table, messages, featureStr, start, total)
+            return (table, " ".join(messages), featureStr, start, total)
 
         def exposed_csvs(self, query, tuples, sections, **options):
             """Gets query results etc. in plain csv format.
@@ -526,19 +545,19 @@ def makeTfKernel(app, appName, port):
                     pass
 
             queryResults = ()
-            queryMessages = ""
+            queryMessages = ("", "")
             features = ()
             if query:
                 (queryResults, queryMessages, features) = runSearch(app, query, cache)
                 (queryResultsC, queryMessagesC, featuresC) = (
                     runSearchCondensed(app, query, cache, condenseType)
-                    if not queryMessages and condensed and condenseType
-                    else (None, None, None)
+                    if not queryMessages[0] and condensed and condenseType
+                    else (None, ("", ""), None)
                 )
 
-                if queryMessages:
+                if queryMessages[0]:
                     queryResults = ()
-                if queryMessagesC:
+                if queryMessagesC[0]:
                     queryResultsC = ()
 
             csvs = (
@@ -564,7 +583,7 @@ def makeTfKernel(app, appName, port):
                 fmt=fmt,
             )
             return (
-                queryMessages,
+                " ".join(queryMessages[0]),
                 pickle.dumps(csvs),
                 pickle.dumps(tupleResultsX),
                 pickle.dumps(queryResultsX),
@@ -584,12 +603,12 @@ def makeTfKernel(app, appName, port):
 # KERNEL CONNECTION
 
 
-def makeTfConnection(host, port, timeout):
+def makeTfConnection(lhost, port, timeout):
     class TfConnection(object):
         def connect(self):
             try:
                 connection = rpyc.connect(
-                    host, port, config=dict(sync_request_timeout=timeout)
+                    lhost, port, config=dict(sync_request_timeout=timeout)
                 )
                 self.connection = connection
             except ConnectionRefusedError as e:
@@ -609,9 +628,11 @@ def main(cargs=sys.argv):
         return
 
     (dataSource, portKernel) = args
+    backend = dataSource.get("backend", GH) or GH
     appName = dataSource["appName"]
     checkout = dataSource["checkout"]
     checkoutApp = dataSource["checkoutApp"]
+    dataLoc = dataSource["dataLoc"]
     moduleRefs = dataSource["moduleRefs"]
     locations = dataSource["locations"]
     modules = dataSource["modules"]
@@ -623,11 +644,14 @@ def main(cargs=sys.argv):
 
     versionRep = "" if version is None else f" version {version}"
     console(
-        f"Setting up TF kernel for {appName} {moduleRefs or ''} {setFile or ''}{versionRep}"
+        f"Setting up TF kernel for {appName} {moduleRefs or ''} "
+        f"{setFile or ''}{versionRep}"
     )
     app = findApp(
         appName,
         checkoutApp,
+        dataLoc,
+        backend,
         True,
         checkout=checkout,
         mod=moduleRefs,
