@@ -4,6 +4,17 @@
  *
  */
 
+// character widget
+
+const copyChar = (el, c) => {
+    for (const el of document.getElementsByClassName('ccon')) {
+        el.className = 'ccoff'
+    }
+    el.className = 'ccon'
+    navigator.clipboard.writeText(String.fromCharCode(c))
+}
+window.copyChar = copyChar
+
 const lastJobKey = 'tfLastJob'
 
 const switchMode = m => {
@@ -72,6 +83,7 @@ const modes = () => {
   }
 }
 
+
 // switch to passage mode after clicking on a result
 
 const switchEvent = e => {
