@@ -35,8 +35,8 @@ from shutil import rmtree
 
 from ..parameters import OTYPE, OSLOTS, OVOLUME, OWORK, OINTERF, OINTERT, OMAP
 from ..core.fabric import FabricCore
-from ..core.timestamp import Timestamp
-from ..core.helpers import dirEmpty, unexpanduser as ux, getAllFeatures
+from ..core.timestamp import Timestamp, SILENT_D, DEEP
+from ..core.helpers import dirEmpty, unexpanduser as ux, getAllRealFeatures
 
 DEBUG = False
 
@@ -65,10 +65,10 @@ def collect(
     volumeFeature=None,
     mergeTypes=None,
     featureMeta=None,
-    silent=False,
+    silent=SILENT_D,
     overwrite=False,
 ):
-    """Creates a work out of a number of volumes.
+    """Creates a collection out of a number of volumes.
 
     The volumes are individual TF data sets, the work is a new TF data set.
 
@@ -226,8 +226,8 @@ def collect(
         and will override anything with the same key
         that is already in one of the volumes.
 
-    silent: boolean, optional `False`
-        Suppress or enable informational messages.
+    silent: string, optional `tf.core.timestamp.SILENT_D`
+        See `tf.core.timestamp.Timestamp`
 
     Returns
     -------
@@ -269,7 +269,7 @@ def collect(
                     sectionFeatures='title,number,number',
                 ),
             ),
-            silent=False,
+            silent=SILENT_D,
         )
 
     This will give rise to something like this (assuming that `banks` and
@@ -425,7 +425,7 @@ def collect(
             meta[volumeFeature]["description"][f"label of {volumeType}"] = {""}
 
         for name in volumes:
-            allFeatures = getAllFeatures(apis[name])
+            allFeatures = getAllRealFeatures(apis[name])
             for (feat, fObj) in TFs[name].features.items():
                 if feat not in allFeatures:
                     continue
@@ -547,6 +547,7 @@ def collect(
         info("Collect nodes from volumes ...")
         indent(level=1, reset=True)
         nodesByType = collections.defaultdict(list)
+        fromWork.clear()
 
         sW = 0
 
@@ -681,10 +682,11 @@ def collect(
         maxNodes = {}
         nodeFeatureDatas = {}
         edgeFeatureDatas = {}
+        fromWork.clear()
 
         for name in volumes:
             api = apis[name]
-            allFeatures = getAllFeatures(api)
+            allFeatures = getAllRealFeatures(api)
             for (ointer, OINTER) in ((ointerf, OINTERF), (ointert, OINTERT)):
                 if api.isLoaded(features=OINTER, pretty=False)[OINTER] is None:
                     continue
@@ -735,16 +737,16 @@ def collect(
             )
 
             if nW > maxSlotW:
-                oslots[nW] = set(
+                oslots[nW] = {
                     volumeMapI[(name, sV)]
                     for sV in eOslotsDatas[name][nV - maxSlotI - 1]
-                )
+                }
                 if nW in fromWork:
                     for (name2, nV2) in fromWork[nW]:
-                        oslots[nW] |= set(
+                        oslots[nW] |= {
                             volumeMapI[(name2, sV)]
                             for sV in eOslotsDatas[name2][nV2 - maxSlots[name2] - 1]
-                        )
+                        }
 
             for (feat, featD) in nodeFeatureDatas[name].items():
                 val = featD.get(nV, None)
@@ -855,12 +857,12 @@ def collect(
 
     def writeTf():
         info("write work as TF data set")
-        TF = FabricCore(locations=workLocation, silent=True)
+        TF = FabricCore(locations=workLocation, silent=DEEP)
         good = TF.save(
             metaData=metaData,
             nodeFeatures=nodeFeatures,
             edgeFeatures=edgeFeatures,
-            silent=False if DEBUG else silent or True,
+            silent=SILENT_D if DEBUG else DEEP,
         )
         indent(level=0)
         if not good:
