@@ -14,13 +14,14 @@ extract(
 
 """
 
+import os
 import collections
 from shutil import rmtree
 
 from ..parameters import OTYPE, OSLOTS, OWORK, OINTERF, OINTERT
 from ..core.fabric import FabricCore
-from ..core.timestamp import Timestamp
-from ..core.helpers import dirEmpty, unexpanduser as ux, getAllFeatures
+from ..core.timestamp import Timestamp, SILENT_D, DEEP, silentConvert
+from ..core.helpers import dirEmpty, unexpanduser as ux, getAllRealFeatures
 
 DEBUG = False
 OWORKI = "oworki"
@@ -35,15 +36,58 @@ setSilent = TM.setSilent
 isSilent = TM.isSilent
 
 
+def getVolumes(volumesLocation):
+    """Lists volumes of a work that exist in a given directory.
+
+    A directory is a volume if it contains the file `otype.tf` and that file
+    has a line starting with `@volume=`*xxx* where *xxx* is the name of the directory.
+
+    Parameters
+    ----------
+    volumesLocation: string
+        The directory to search for volumes.
+
+    Returns
+    -------
+    None or list
+        If `volumesLocation` does not exist or is not a directory, None
+        is returned. Otherwise a list of subdirectories that are modules.
+    """
+
+    if not os.path.isdir(volumesLocation):
+        return None
+
+    volumes = []
+
+    with os.scandir(volumesLocation) as dh:
+        for entry in dh:
+            vol = entry.name
+            if entry.is_dir() and not vol.startswith("."):
+                triggerFile = f"{volumesLocation}/{vol}/{OTYPE}.tf"
+                triggerLine = f"@volume={vol}\n"
+                if os.path.isfile(triggerFile):
+                    with open(triggerFile) as fh:
+                        triggered = False
+                        for line in fh:
+                            if line == triggerLine:
+                                triggered = True
+                                break
+                    if triggered:
+                        volumes.append(vol)
+
+    return sorted(volumes)
+
+
 def extract(
     workLocation,
     volumesLocation,
-    volumes,
+    volumes=True,
     byTitle=True,
-    silent=False,
+    silent=SILENT_D,
     api=None,
     overwrite=None,
     checkOnly=False,
+    show=False,
 ):
     """Extracts volumes of a work.
 
@@ -57,7 +101,7 @@ def extract(
     Volumes will get a node feature `owork` which maps nodes in the volume to
     nodes in the work.
 
-    !!! note "use of feature owork
+    !!! note "use of feature owork"
         If volumes are combined to a work, nodes in distinct volumes may
         correspond to a single node in the work. In that case, they have the
         same value in the `owork` feature.
@@ -119,7 +163,7 @@ def extract(
         The directory under which the feature files of the volumes
         will be written.
 
-    volumes: boolean or dict or set
+    volumes: boolean or dict or set, optional True
         If True, extracts all top-level sections into separate volumes.
 
         If it is a dict, the keys are names for the volumes, and the values
@@ -146,8 +190,8 @@ def extract(
         Default: by their titles.
         Note, that depending on the work, section titles may be strings or integers.
 
-    silent: boolean, optional `False`
-        Suppress or enable informational messages.
+    silent: string, optional `tf.core.timestamp.SILENT_D`
+        See `tf.core.timestamp.Timestamp`
 
     api: object, optional `None`
         If given, assume it is the TF api of a loaded work
@@ -169,6 +213,9 @@ def extract(
         If there is an error, returns False, otherwise returns the volumes in as
         far as they have to be extracted.
 
+    show: boolean, optional False
+        If True, does not return anything, but pretty prints the result
+        to the screen.
 
     Returns
     -------
@@ -228,14 +275,18 @@ def extract(
     `thora` with the first 5 books and `poetry` with two poetic books.
     """
 
+    silent = silentConvert(silent)
+
+    givenVolumes = set()
     volumeData = {}
 
     def checkVolumeLocs():
         good = True
 
         removable = set()
-        for name in volumeData:
+        for (name, v) in volumeData.items():
             loc = f"{volumesLocation}/{name}"
+            v["location"] = loc
             if not dirEmpty(loc):
                 if overwrite is None:
                     info(
@@ -263,14 +314,19 @@ def extract(
 
         good = True
         if volumes is not True:
+            givenVolumes.clear()
+
             if type(volumes) is dict:
                 for (name, heads) in volumes.items():
                     volumeData[name] = dict(heads=heads)
+                    givenVolumes.add(name)
             else:
                 for heads in volumes:
-                    volumeData["-".join(str(head) for head in heads)] = dict(
+                    headRep = "-".join(str(head) for head in heads)
+                    volumeData[headRep] = dict(
                         heads=heads
                     )
+                    givenVolumes.add(headRep)
 
             headIndex = {}
 
@@ -294,9 +350,11 @@ def extract(
         return good
 
     def checkVolumes2():
+        givenVolumes.clear()
         if volumes is True:
             for head in toplevels:
                 volumeData[head] = dict(heads=(head,))
+                givenVolumes.add(head)
             if not checkVolumeLocs():
                 return False
 
@@ -357,7 +415,7 @@ def extract(
     fOtypeData = F.otype.data
     eOslotsData = E.oslots.data
 
-    allFeatures = getAllFeatures(api)
+    allFeatures = getAllRealFeatures(api)
 
     metaDataTotal = {feat: TF.features[feat].metaData for feat in allFeatures}
     nodeFeatureData = {
@@ -482,7 +540,7 @@ def extract(
             metaData = {}
             v["metaData"] = metaData
             headStr = "-".join(str(head) for head in v["heads"])
-            volumeMeta = name if headStr == name else f"{name}:{headStr}"
+            volumeMeta = name if headStr == str(name) else f"{name}:{headStr}"
 
             for (feat, meta) in metaDataTotal.items():
                 metaData[feat] = {k: m for (k, m) in meta.items()}
@@ -605,13 +663,13 @@ def extract(
             nodeFeatures = v["nodeFeatures"]
             edgeFeatures = v["edgeFeatures"]
             loc = f"{volumesLocation}/{name}"
-            v["loc"] = loc
-            TF = FabricCore(locations=loc, silent=True)
+            v["location"] = loc
+            TF = FabricCore(locations=loc, silent=DEEP)
             if not TF.save(
                 metaData=metaData,
                 nodeFeatures=nodeFeatures,
                 edgeFeatures=edgeFeatures,
-                silent=False if DEBUG else silent or True,
+                silent=SILENT_D if DEBUG else DEEP,
             ):
                 good = False
         indent(level=0)
@@ -623,30 +681,52 @@ def extract(
     def process():
         indent(level=0, reset=True)
         if not checkVolumes():
-            return False
+            return None if show else False
         if checkOnly:
-            if not volumeData:
-                return {name: v["heads"] for (name, v) in volumeData.items()}
+            if volumeData:
+                if show:
+                    for (name, v) in volumeData.items():
+                        location = ux(v['location'])
+                        print(f"{name:<20} @ {location}")
+                else:
+                    return volumeData
+            else:
+                if show:
+                    print("No volumes")
+                else:
+                    return {}
         if not getTopLevels():
-            return False
+            return None if show else False
         if not checkVolumes2():
-            return False
+            return None if show else False
         if checkOnly:
-            return {name: v["heads"] for (name, v) in volumeData.items()}
+            if show:
+                for (name, v) in volumeData.items():
+                    location = ux(v['location'])
+                    print(f"{name:<20} @ {location}")
+            else:
+                return volumeData
         if not distributeNodes():
-            return False
+            return None if show else False
         if not remapFeatures():
-            return False
+            return None if show else False
         if not writeTf():
-            return False
+            return None if show else False
         info("All done")
-        return {
+        result = {
             name: dict(location=f"{volumesLocation}/{name}", new=name in volumeData)
-            for name in volumes
+            for name in sorted(givenVolumes | set(volumeData))
         }
+        if show:
+            for (name, v) in result.items():
+                new = " (new)" if v["new"] else " " * 6
+                location = ux(v['location'])
+                print(f"{name:<20}{new} @ {location}")
+        else:
+            return result
 
     wasSilent = isSilent()
     setSilent(silent)
     result = process()
     setSilent(wasSilent)
-    return result
+    return None if show else result
